from concurrent.futures import Future
from typing import TYPE_CHECKING, Literal, Optional, Union, overload

from pydantic import StrictStr

from snowflake.core import FQN, PollingOperation
from snowflake.core._common import (
    Clone,
    CreateMode,
    PointOfTime,
)

from .._internal.telemetry import api_telemetry
from .._internal.utils import deprecated
from ._generated import SuccessResponse
from ._generated.api.dynamic_table_api_base import DynamicTableCollectionBase, DynamicTableResourceBase
from ._generated.models.dynamic_table import DynamicTable
from ._generated.models.dynamic_table_clone import DynamicTableClone
from ._generated.models.point_of_time import PointOfTime as TablePointOfTime


if TYPE_CHECKING:
    from snowflake.core.schema import SchemaResource


class DynamicTableCollection(DynamicTableCollectionBase):
    """Represents the collection operations on the Snowflake Dynamic Table resource.

    With this collection, you can create, iterate through, and search for dynamic tables that you have access to in the
    current context.

    Examples
    ________
    Creating a dynamic table instance:

    >>> dynamic_tables = root.databases["my_db"].schemas["my_schema"].dynamic_tables
    >>> dynamic_tables.create(
    ...     DynamicTable(
    ...         name="my_dynamic_table",
    ...         columns=[
    ...             DynamicTableColumn(name="c1"),
    ...             DynamicTableColumn(name='"cc2"', datatype="varchar"),
    ...         ],
    ...         warehouse=db_parameters["my_warehouse"],
    ...         target_lag=UserDefinedLag(seconds=60),
    ...         query="SELECT * FROM my_table",
    ...     ),
    ...     mode=CreateMode.error_if_exists,
    ... )
    """

    def __init__(self, schema: "SchemaResource"):
        super().__init__(schema, DynamicTableResource)

    @api_telemetry
    def create(  # type: ignore[override]
        self,
        table: Union[DynamicTable, DynamicTableClone, str],
        *,
        clone_table: Optional[Union[str, Clone]] = None,
        copy_grants: Optional[bool] = False,
        mode: CreateMode = CreateMode.error_if_exists,
    ) -> "DynamicTableResource":
        """Create a dynamic table.

        Parameters
        __________
        table: DynamicTable | DynamicTableClone | str
            1. The ``DynamicTable`` object, together with the dynamic table's properties:
                name, target_lag, warehouse, query;
                columns, refresh_mode, initialize, cluster_by, comment are optional.
            2. The ``DynamicTableClone`` object, when it's used with `clone_table`.
            3. The table name.
        clone_table: Clone, optional
            The source table to clone from.
        copy_grants: bool, optional
            Whether to enable copy grants when creating the object. Default is ``False``.
        mode: CreateMode, optional
            One of the following enum values.

            ``CreateMode.error_if_exists``: Throw an :class:`snowflake.core.exceptions.ConflictError`
            if the dynamic table already exists in Snowflake.  Equivalent to SQL ``create dynamic table <name> ...``.

            ``CreateMode.or_replace``: Replace if the dynamic table already exists in Snowflake. Equivalent to SQL
            ``create or replace dynamic table <name> ...``.

            ``CreateMode.if_not_exists``: Do nothing if the dynamic table already exists in Snowflake.
            Equivalent to SQL ``create dynamic table <name> if not exists...``

            Default is ``CreateMode.error_if_exists``.

        Examples
        ________
        Creating a dynamic table, replacing any existing dynamic table with the same name:

        >>> dynamic_tables = root.databases["my_db"].schemas["my_schema"].dynamic_tables
        >>> dynamic_tables.create(
        ...     DynamicTable(
        ...         name="my_dynamic_table",
        ...         columns=[
        ...             DynamicTableColumn(name="c1"),
        ...             DynamicTableColumn(name='"cc2"', datatype="varchar"),
        ...         ],
        ...         warehouse=db_parameters["my_warehouse"],
        ...         target_lag=UserDefinedLag(seconds=60),
        ...         query="SELECT * FROM my_table",
        ...     ),
        ...     mode=CreateMode.error_if_exists,
        ... )

        Creating a dynamic table by cloning an existing table:

        >>> dynamic_tables = root.databases["my_db"].schemas["my_schema"].dynamic_tables
        >>> dynamic_tables.create(
        ...     DynamicTableClone(
        ...         name="my_dynamic_table",
        ...         target_lag=UserDefinedLag(seconds=120),
        ...     ),
        ...     clone_table=Clone(
        ...         source="my_source_dynamic_table",
        ...         point_of_time=PointOfTimeOffset(reference="before", when="-1"),
        ...     ),
        ...     copy_grants=True,
        ... )

        Creating a dynamic table by cloning an existing table in a different database and schema:

        >>> dynamic_tables = root.databases["my_db"].schemas["my_schema"].dynamic_tables
        >>> dynamic_tables.create(
        ...     DynamicTableClone(
        ...         name="my_dynamic_table",
        ...         target_lag=UserDefinedLag(seconds=120),
        ...     ),
        ...     clone_table=Clone(
        ...         source="database_of_source_table.schema_of_source_table.my_source_dynamic_table",
        ...         point_of_time=PointOfTimeOffset(reference="before", when="-1"),
        ...     ),
        ...     copy_grants=True,
        ... )
        """
        self._create(
            table=table,
            clone_table=clone_table,
            copy_grants=copy_grants,
            mode=mode,
            async_req=False,
        )
        return DynamicTableResource(table if isinstance(table, str) else table.name, self)

    @api_telemetry
    def create_async(  # type: ignore[override]
        self,
        table: Union[DynamicTable, DynamicTableClone, str],
        *,
        clone_table: Optional[Union[str, Clone]] = None,
        copy_grants: Optional[bool] = False,
        mode: CreateMode = CreateMode.error_if_exists,
    ) -> PollingOperation["DynamicTableResource"]:
        """An asynchronous version of :func:`create`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self._create(
            table=table,
            clone_table=clone_table,
            copy_grants=copy_grants,
            mode=mode,
            async_req=True,
        )
        return PollingOperation(
            future, lambda _: DynamicTableResource(table if isinstance(table, str) else table.name, self)
        )

    @overload
    def _create(
        self,
        table: Union[DynamicTable, DynamicTableClone, str],
        clone_table: Optional[Union[str, Clone]],
        copy_grants: Optional[bool],
        mode: CreateMode,
        async_req: Literal[True],
    ) -> Future[SuccessResponse]: ...

    @overload
    def _create(
        self,
        table: Union[DynamicTable, DynamicTableClone, str],
        clone_table: Optional[Union[str, Clone]],
        copy_grants: Optional[bool],
        mode: CreateMode,
        async_req: Literal[False],
    ) -> SuccessResponse: ...

    def _create(
        self,
        table: Union[DynamicTable, DynamicTableClone, str],
        clone_table: Optional[Union[str, Clone]],
        copy_grants: Optional[bool],
        mode: CreateMode,
        async_req: bool,
    ) -> Union[SuccessResponse, Future[SuccessResponse]]:
        real_mode = CreateMode[mode].value

        if clone_table:
            # create table by clone

            if isinstance(table, str):
                table = DynamicTableClone(name=table)

            pot: Optional[TablePointOfTime] = None
            if isinstance(clone_table, Clone) and isinstance(clone_table.point_of_time, PointOfTime):
                pot = TablePointOfTime.from_dict(clone_table.point_of_time.to_dict())
            real_clone = Clone(source=clone_table) if isinstance(clone_table, str) else clone_table
            req = DynamicTableClone(
                name=table.name,
                target_lag=table.target_lag,
                warehouse=table.warehouse,
                point_of_time=pot,
            )

            source_table_fqn = FQN.from_string(real_clone.source)
            return self._api.clone_dynamic_table(
                source_table_fqn.database or self.database.name,
                source_table_fqn.schema or self.schema.name,
                source_table_fqn.name,
                req,
                create_mode=StrictStr(real_mode),
                copy_grants=copy_grants,
                target_database=self.database.name,
                target_schema=self.schema.name,
                async_req=async_req,
            )

        # create empty table

        if not isinstance(table, DynamicTable):
            raise ValueError("`table` must be a `DynamicTable` unless `clone_table` is used")

        return self._api.create_dynamic_table(
            self.database.name,
            self.schema.name,
            table,
            create_mode=StrictStr(real_mode),
            async_req=async_req,
        )


class DynamicTableResource(DynamicTableResourceBase):
    """Represents a reference to a Snowflake dynamic table.

    With this dynamic table reference, you can fetch information about a dynamic table, as well
    as perform certain actions on it.
    """

    _plural_name = "dynamic_tables"

    @api_telemetry
    @deprecated("drop")
    def delete(self) -> None:
        self.drop()

    @api_telemetry
    @deprecated("undrop")
    def undelete(self) -> None:
        self.undrop()

    @api_telemetry
    def swap_with(  # type: ignore[override]
        self,
        to_swap_table_name: str,
        if_exists: Optional[bool] = None,
    ) -> None:
        """Swap the name with another dynamic table.

        Parameters
        __________
        to_swap_table_name: str
            The name of the table to swap with.
        if_exists: bool, optional
            Check the existence of this dynamic table before swapping its name.
            Default is ``None``, which is equivalent to ``False``.

        Examples
        ________
        Swaping name with another dynamic table using its reference:
        >>> dynamic_table_reference.swap_with("my_other_dynamic_table")
        """
        super().swap_with(
            target_name=to_swap_table_name,
            # target_schema=self.schema.name,
            # target_database=self.database.name,
            if_exists=if_exists,
        )

    @api_telemetry
    def swap_with_async(  # type: ignore[override]
        self,
        to_swap_table_name: str,
        if_exists: Optional[bool] = None,
    ) -> PollingOperation[None]:
        """An asynchronous version of :func:`swap_with`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        return super().swap_with_async(
            target_name=to_swap_table_name,
            # target_schema=self.schema.name,
            # target_database=self.database.name,
            if_exists=if_exists,
        )



from unittest import mock

import pytest

from snowflake.core import PollingOperation
from snowflake.core.network_rule import NetworkRule, NetworkRuleResource

from ...utils import BASE_URL, extra_params, mock_http_response


@pytest.fixture
def network_rules(schema):
    return schema.network_rules


@pytest.fixture
def network_rule(network_rules):
    return network_rules["my_network_rule"]


API_CLIENT_REQUEST = "snowflake.core._generated.api_client.ApiClient.request"
NETWORK_RULE = NetworkRule(
    name="my_network_rule",
    type="HOST_PORT",
    mode="EGRESS",
    value_list=["example.com:443", "api.example.com:443"],
    comment="Test network rule",
)


def test_create_network_rule(fake_root, network_rules):
    args = (fake_root, "POST", BASE_URL + "/databases/my_db/schemas/my_schema/network-rules")
    kwargs = extra_params(
        query_params=[],
        body={
            "type": "HOST_PORT",
            "mode": "EGRESS",
            "value_list": ["example.com:443", "api.example.com:443"],
            "comment": "Test network rule",
            "name": "my_network_rule",
        },
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        nr_res = network_rules.create(NETWORK_RULE)
        assert isinstance(nr_res, NetworkRuleResource)
        assert nr_res.name == "my_network_rule"
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = network_rules.create_async(NETWORK_RULE)
        assert isinstance(op, PollingOperation)
        nr_res = op.result()
        assert nr_res.name == "my_network_rule"
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_network_rule(fake_root, network_rules):
    args = (fake_root, "GET", BASE_URL + "/databases/my_db/schemas/my_schema/network-rules")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        network_rules.iter()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = network_rules.iter_async()
        assert isinstance(op, PollingOperation)
        it = op.result()
        assert list(it) == []
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_show_limit_deprecation_warning(fake_root, network_rules):
    args = (
        fake_root,
        "GET",
        BASE_URL + "/databases/my_db/schemas/my_schema/network-rules?showLimit=10",
    )
    kwargs = extra_params(query_params=[("showLimit", 10)])

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        with pytest.warns(DeprecationWarning, match="'show_limit' is deprecated, use 'limit' instead"):
            list(network_rules.iter(show_limit=10))
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        with pytest.warns(DeprecationWarning, match="'show_limit' is deprecated, use 'limit' instead"):
            network_rules.iter_async(show_limit=10).result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_limit_and_show_limit_conflict(network_rules):
    with pytest.raises(ValueError, match="Cannot specify both 'limit' and 'show_limit'"):
        list(network_rules.iter(limit=10, show_limit=5))

    with pytest.raises(ValueError, match="Cannot specify both 'limit' and 'show_limit'"):
        network_rules.iter_async(limit=10, show_limit=5).result()


def test_fetch_network_rule(fake_root, network_rule):
    args = (fake_root, "GET", BASE_URL + "/databases/my_db/schemas/my_schema/network-rules/my_network_rule")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(NETWORK_RULE.to_json())
        network_rule.fetch()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(NETWORK_RULE.to_json())
        op = network_rule.fetch_async()
        assert isinstance(op, PollingOperation)
        rule = op.result()
        assert rule.to_dict() == NETWORK_RULE.to_dict()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_drop_network_rule(fake_root, network_rule):
    args = (fake_root, "DELETE", BASE_URL + "/databases/my_db/schemas/my_schema/network-rules/my_network_rule")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        network_rule.drop()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = network_rule.drop_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)



"""
Snowflake Catalog Integration API.

The Snowflake Catalog Integration API is a REST API that you can use to access, update, and perform certain actions on Catalog Integration resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any

from pydantic import ConfigDict, StrictStr

from snowflake.core.catalog_integration._generated.models.catalog import Catalog
from snowflake.core.catalog_integration._generated.models.rest_authentication import (
    RestAuthentication,
    RestAuthenticationModel,
)
from snowflake.core.catalog_integration._generated.models.rest_config import RestConfig, RestConfigModel


class Polaris(Catalog):
    """A model object representing the Polaris resource.

    Constructs an object of type Polaris with the provided properties.

    Parameters
    __________
    catalog_namespace : str
        Default Polaris namespace used by all Iceberg tables associated with this catalog integration
    rest_config : RestConfig

    rest_authentication : RestAuthentication
    """

    catalog_namespace: StrictStr

    rest_config: RestConfig

    rest_authentication: RestAuthentication

    __properties = ["catalog_source"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Polaris:
        """Create an instance of Polaris from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of rest_config
        if self.rest_config:
            _dict["rest_config"] = self.rest_config.to_dict()

        # override the default output from pydantic by calling `to_dict()` of rest_authentication
        if self.rest_authentication:
            _dict["rest_authentication"] = self.rest_authentication.to_dict()

        _dict["catalog_source"] = Catalog.get_child_model_discriminator_value("Polaris")

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> Polaris:
        """Create an instance of Polaris from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return Polaris.model_validate(obj)

        _obj = Polaris.model_validate(
            {
                "catalog_namespace": obj.get("catalog_namespace"),
                "rest_config": RestConfig.from_dict(obj.get("rest_config"))
                if obj.get("rest_config") is not None
                else None,
                "rest_authentication": RestAuthentication.from_dict(obj.get("rest_authentication"))
                if obj.get("rest_authentication") is not None
                else None,
            }
        )

        return _obj


class PolarisModel(Catalog):
    def __init__(
        self,
        catalog_namespace: str,
        rest_config: RestConfig,
        rest_authentication: RestAuthentication,
        # optional properties
    ):
        """A model object representing the Polaris resource.

        Constructs an object of type Polaris with the provided properties.

        Parameters
        __________
        catalog_namespace : str
            Default Polaris namespace used by all Iceberg tables associated with this catalog integration
        rest_config : RestConfig

        rest_authentication : RestAuthentication
        """
        super().__init__()
        self.catalog_namespace = catalog_namespace
        self.rest_config = rest_config
        self.rest_authentication = rest_authentication

    __properties = ["catalog_source"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return Polaris(
            catalog_namespace=self.catalog_namespace,
            rest_config=self.rest_config._to_model() if self.rest_config is not None else None,
            rest_authentication=self.rest_authentication._to_model() if self.rest_authentication is not None else None,
        )

    @classmethod
    def _from_model(cls, model) -> PolarisModel:
        return PolarisModel(
            catalog_namespace=model.catalog_namespace,
            rest_config=RestConfigModel._from_model(model.rest_config) if model.rest_config else None,
            rest_authentication=RestAuthenticationModel._from_model(model.rest_authentication)
            if model.rest_authentication
            else None,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a Polaris.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the Polaris object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> PolarisModel:
        """Create an instance of Polaris from a dict.

        This method constructs a Polaris object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        Polaris
            A Polaris object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(Polaris.from_dict(obj))


Polaris._model_class = PolarisModel



from snowflake.core.artifact_repository import ArtifactRepository
from tests.integ.utils import random_string


def test_create_or_alter_artifact_repository(artifact_repositories, temp_pypi_api_integration):
    """Test create_or_alter functionality - creating new artifact repository."""
    ar_name = random_string(10, "test_artifact_repository_create_or_alter_")
    ar_def = ArtifactRepository(
        name=ar_name,
        type="PIP",
        api_integration=temp_pypi_api_integration,
        comment="created by test_create_or_alter_artifact_repository",
    )
    ar = artifact_repositories[ar_name]
    ar.create_or_alter(ar_def)
    try:
        fetched_ar = ar.fetch()
        assert fetched_ar.name == ar_name.upper()
        assert fetched_ar.type == ar_def.type
        assert fetched_ar.api_integration == temp_pypi_api_integration
        assert fetched_ar.comment == ar_def.comment

        fetched_ar.comment = "altered by test_create_or_alter_artifact_repository"
        ar.create_or_alter(fetched_ar)

        altered_ar = ar.fetch()
        assert altered_ar.name == ar_name.upper()
        assert altered_ar.type == ar_def.type
        assert altered_ar.api_integration == temp_pypi_api_integration
        assert altered_ar.comment == "altered by test_create_or_alter_artifact_repository"
    finally:
        ar.drop()


def test_create_or_alter_unset_comment(artifact_repositories, temp_pypi_api_integration):
    """Test create_or_alter unsetting comment field."""
    ar_name = random_string(10, "test_artifact_repository_unset_comment_")
    ar_def = ArtifactRepository(
        name=ar_name,
        type="PIP",
        api_integration=temp_pypi_api_integration,
        comment="comment to be removed",
    )
    ar = artifact_repositories.create(ar_def)
    try:
        updated_ar_def = ArtifactRepository(
            name=ar_name,
            type="PIP",
            api_integration=temp_pypi_api_integration,
            comment=None,
        )
        ar.create_or_alter(updated_ar_def)

        fetched_ar = ar.fetch()
        assert fetched_ar.name == ar_name.upper()
        assert fetched_ar.comment is None
    finally:
        ar.drop()



import copy

from typing import Iterator

import pytest

from snowflake.core import CreateMode
from snowflake.core.exceptions import ConflictError, NotFoundError
from snowflake.core.sequence import Sequence, SequenceResource
from tests.integ.utils import random_string

from .conftest import test_sequence_template


@pytest.fixture
def temp_sequence(sequences) -> Iterator[SequenceResource]:
    sequence_name = random_string(10, "test_sequence_clone_original_")
    sequence = copy.deepcopy(test_sequence_template)
    sequence.name = sequence_name
    sequence_handle = sequences.create(sequence)

    try:
        yield sequence_handle
    finally:
        sequences[sequence_name].drop(if_exists=True)


def test_clone(sequences, temp_sequence):
    clone_name = random_string(10, "test_sequence_clone_")
    clone_sequence = Sequence(name=clone_name, comment="Updated comment")

    try:
        temp_sequence.clone(clone_sequence)

        fetched_clone = sequences[clone_name].fetch()
        assert fetched_clone.name == clone_name.upper()
        assert fetched_clone.start == 2
        assert fetched_clone.increment == 2
        assert fetched_clone.ordered is True
        assert fetched_clone.comment == "Updated comment"
        assert fetched_clone.schema_name == sequences.schema.name.upper()
        assert fetched_clone.database_name == sequences.database.name.upper()
    finally:
        sequences[clone_name].drop(if_exists=True)


def test_clone_nonexistent_sequence(sequences):
    nonexistent_name = random_string(10, "test_sequence_clone_nonexistest_")
    clone_name = random_string(10, "test_sequence_clone_")
    clone_sequence = Sequence(name=clone_name)

    with pytest.raises(NotFoundError):
        sequences[nonexistent_name].clone(clone_sequence)


def test_clone_create_modes(sequences, temp_sequence):
    clone_name = random_string(10, "test_sequence_clone_modes_")
    clone_sequence = Sequence(name=clone_name, comment="First version")

    try:
        temp_sequence.clone(clone_sequence, create_mode=CreateMode.error_if_exists)

        assert sequences[clone_name].fetch().comment == "First version"

        with pytest.raises(ConflictError):
            temp_sequence.clone(clone_sequence, create_mode=CreateMode.error_if_exists)

        clone_sequence.comment = "Second version"
        temp_sequence.clone(clone_sequence, create_mode=CreateMode.or_replace)
        assert sequences[clone_name].fetch().comment == "Second version"

        clone_sequence.comment = "Should not change"
        temp_sequence.clone(clone_sequence, create_mode=CreateMode.if_not_exists)
        assert sequences[clone_name].fetch().comment == "Second version"
    finally:
        sequences[clone_name].drop(if_exists=True)


def test_clone_across_schema(temp_sequence, temp_schema):
    clone_name = random_string(10, "test_sequence_clone_cross_schema_")
    clone_sequence = Sequence(name=clone_name)

    try:
        temp_sequence.clone(clone_sequence, target_schema=temp_schema.name)

        cloned_handle = temp_schema.sequences[clone_name]
        fetched_clone = cloned_handle.fetch()

        assert fetched_clone.name == clone_name.upper()
        assert fetched_clone.schema_name == temp_schema.name.upper()
        assert fetched_clone.database_name == temp_sequence.database.name.upper()
    finally:
        temp_schema.sequences[clone_name].drop(if_exists=True)


def test_clone_across_database(temp_sequence, databases, temp_db):
    clone_name = random_string(10, "test_sequence_clone_cross_db_")
    clone_sequence = Sequence(name=clone_name)

    try:
        temp_sequence.clone(clone_sequence, target_database=temp_db.name, target_schema="PUBLIC")

        cloned_handle = databases[temp_db.name].schemas["PUBLIC"].sequences[clone_name]
        fetched_clone = cloned_handle.fetch()

        assert fetched_clone.name == clone_name.upper()
        assert fetched_clone.schema_name == "PUBLIC"
        assert fetched_clone.database_name == temp_db.name.upper()
    finally:
        databases[temp_db.name].schemas["PUBLIC"].sequences[clone_name].drop()



# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.


from contextlib import suppress

import pytest

from snowflake.core._internal.utils import normalize_and_unquote_name
from snowflake.core.exceptions import NotFoundError
from snowflake.core.warehouse import Warehouse

from ..utils import random_string


pytestmark = pytest.mark.usefixtures("backup_warehouse_fixture")


def test_iter(warehouses):
    try:
        warehouse_name_1 = random_string(5, "test_warehouse_")
        warehouse_name_1 = f'"{warehouse_name_1}"'
        warehouse_name_2 = random_string(5, "test_warehouse_A")
        warehouse_name_3 = random_string(5, "test_warehouse_B")
        warehouse_refs = []
        warehouse_refs.append(warehouses.create(warehouse=Warehouse(name=warehouse_name_1)))
        warehouse_refs.append(warehouses.create(warehouse=Warehouse(name=warehouse_name_3)))
        warehouse_refs.append(warehouses.create(warehouse=Warehouse(name=warehouse_name_2)))
        test_warehouses = warehouses.iter(like="%test_warehouse_%")

        expected_warehouse_names = [
            normalize_and_unquote_name(warehouse_name_1),
            warehouse_name_2.upper(),
            warehouse_name_3.upper(),
        ]

        warehouse_list = []
        for warehouse in test_warehouses:
            if warehouse.name in expected_warehouse_names:
                warehouse_list.append(warehouse.name)

        assert len(warehouse_list) == 3
    finally:
        with suppress(NotFoundError):
            for warehouse_ref in warehouse_refs:
                warehouse_ref.drop()



from unittest import mock

import pytest

from snowflake.core import PollingOperation
from snowflake.core.artifact_repository import ArtifactRepository, ArtifactRepositoryResource

from ...utils import BASE_URL, extra_params, mock_http_response


API_CLIENT_REQUEST = "snowflake.core._generated.api_client.ApiClient.request"
ARTIFACT_REPOSITORY = ArtifactRepository(
    name="my_artifact_repository", type="PIP", api_integration="my_api_integration"
)

ARTIFACT_REPOSITORY_WITH_COMMENT = ArtifactRepository(
    name="my_artifact_repository_with_comment",
    type="PIP",
    api_integration="my_api_integration",
    comment="Test comment for artifact repository",
)


@pytest.fixture
def artifact_repositories(schema):
    return schema.artifact_repositories


@pytest.fixture
def artifact_repository(artifact_repositories):
    return artifact_repositories["my_artifact_repository"]


def test_create_artifact_repository(fake_root, artifact_repositories):
    args = (
        fake_root,
        "POST",
        BASE_URL + "/databases/my_db/schemas/my_schema/artifact-repositories",
    )
    kwargs = extra_params(
        query_params=[],
        body={"name": "my_artifact_repository", "type": "PIP", "api_integration": "my_api_integration"},
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        artifact_repo_res = artifact_repositories.create(ARTIFACT_REPOSITORY)
        assert isinstance(artifact_repo_res, ArtifactRepositoryResource)
        assert artifact_repo_res.name == "my_artifact_repository"
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = artifact_repositories.create_async(ARTIFACT_REPOSITORY)
        assert isinstance(op, PollingOperation)
        artifact_repo_res = op.result()
        assert artifact_repo_res.name == "my_artifact_repository"
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_create_or_alter_artifact_repository(fake_root, artifact_repositories):
    args = (
        fake_root,
        "PUT",
        BASE_URL + "/databases/my_db/schemas/my_schema/artifact-repositories/my_artifact_repository",
    )
    kwargs = extra_params(
        body={"name": "my_artifact_repository", "type": "PIP", "api_integration": "my_api_integration"},
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        result = artifact_repositories["my_artifact_repository"].create_or_alter(ARTIFACT_REPOSITORY)
        assert result is None  # create_or_alter returns None
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = artifact_repositories["my_artifact_repository"].create_or_alter_async(ARTIFACT_REPOSITORY)
        assert isinstance(op, PollingOperation)
        result = op.result()
        assert result is None  # create_or_alter_async also returns None
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_artifact_repository(fake_root, artifact_repositories):
    args = (fake_root, "GET", BASE_URL + "/databases/my_db/schemas/my_schema/artifact-repositories")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        artifact_repositories.iter()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = artifact_repositories.iter_async()
        assert isinstance(op, PollingOperation)
        it = op.result()
        assert list(it) == []
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_show_limit_deprecation_warning(fake_root, artifact_repositories):
    """Test that show_limit parameter triggers deprecation warning."""
    args = (
        fake_root,
        "GET",
        BASE_URL + "/databases/my_db/schemas/my_schema/artifact-repositories?showLimit=10",
    )
    kwargs = extra_params(query_params=[("showLimit", 10)])

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        with pytest.warns(DeprecationWarning, match="'show_limit' is deprecated, use 'limit' instead"):
            list(artifact_repositories.iter(show_limit=10))
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        with pytest.warns(DeprecationWarning, match="'show_limit' is deprecated, use 'limit' instead"):
            artifact_repositories.iter_async(show_limit=10).result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_show_limit(fake_root, artifact_repositories):
    args = (
        fake_root,
        "GET",
        BASE_URL + "/databases/my_db/schemas/my_schema/artifact-repositories?showLimit=10",
    )
    kwargs = extra_params(query_params=[("showLimit", 10)])

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        list(artifact_repositories.iter(limit=10))
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        artifact_repositories.iter_async(limit=10).result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_limit_and_show_limit_conflict(artifact_repositories):
    """Test that providing both limit and show_limit raises ValueError."""
    with pytest.raises(ValueError, match="Cannot specify both 'limit' and 'show_limit'"):
        list(artifact_repositories.iter(limit=10, show_limit=5))

    with pytest.raises(ValueError, match="Cannot specify both 'limit' and 'show_limit'"):
        artifact_repositories.iter_async(limit=10, show_limit=5).result()


def test_iter_with_like_filter(fake_root, artifact_repositories):
    args = (
        fake_root,
        "GET",
        BASE_URL + "/databases/my_db/schemas/my_schema/artifact-repositories?like=test_repo",
    )
    kwargs = extra_params(query_params=[("like", "test_repo")])

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        list(artifact_repositories.iter(like="test_repo"))
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_with_starts_with_filter(fake_root, artifact_repositories):
    args = (
        fake_root,
        "GET",
        BASE_URL + "/databases/my_db/schemas/my_schema/artifact-repositories?startsWith=test",
    )
    kwargs = extra_params(query_params=[("startsWith", "test")])

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        list(artifact_repositories.iter(starts_with="test"))
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_with_from_name(fake_root, artifact_repositories):
    args = (
        fake_root,
        "GET",
        BASE_URL + "/databases/my_db/schemas/my_schema/artifact-repositories?fromName=start_repo",
    )
    kwargs = extra_params(query_params=[("fromName", "start_repo")])

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        list(artifact_repositories.iter(from_name="start_repo"))
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_fetch_artifact_repository(fake_root, artifact_repository):
    from snowflake.core.artifact_repository._generated.models import ArtifactRepository as ArtifactRepositoryModel

    model = ArtifactRepositoryModel(name="my_artifact_repository", type="PIP", api_integration="my_api_integration")
    args = (
        fake_root,
        "GET",
        BASE_URL + "/databases/my_db/schemas/my_schema/artifact-repositories/my_artifact_repository",
    )
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(model.to_json())
        artifact_repository.fetch()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(model.to_json())
        op = artifact_repository.fetch_async()
        assert isinstance(op, PollingOperation)
        fetched_repo = op.result()
        assert fetched_repo.to_dict() == ARTIFACT_REPOSITORY.to_dict()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_drop_artifact_repository(fake_root, artifact_repository):
    args = (
        fake_root,
        "DELETE",
        BASE_URL + "/databases/my_db/schemas/my_schema/artifact-repositories/my_artifact_repository",
    )
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        artifact_repository.drop()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = artifact_repository.drop_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_drop_artifact_repository_if_exists(fake_root, artifact_repository):
    args = (
        fake_root,
        "DELETE",
        BASE_URL + "/databases/my_db/schemas/my_schema/artifact-repositories/my_artifact_repository?ifExists=True",
    )
    kwargs = extra_params(query_params=[("ifExists", True)])

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        artifact_repository.drop(if_exists=True)
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_rename_artifact_repository(fake_root, artifact_repository):
    args = (
        fake_root,
        "POST",
        BASE_URL
        + "/databases/my_db/schemas/my_schema/artifact-repositories/my_artifact_repository:rename?targetName=new_name",
    )
    kwargs = extra_params(query_params=[("targetName", "new_name")])

    with (
        mock.patch(API_CLIENT_REQUEST) as mocked_request,
        mock.patch.object(artifact_repository, "_rename_finalizer") as mock_finalizer,
    ):
        artifact_repository.rename("new_name")
        mock_finalizer.assert_called_once()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with (
        mock.patch(API_CLIENT_REQUEST) as mocked_request,
        mock.patch.object(artifact_repository, "_rename_finalizer") as mock_finalizer,
    ):
        op = artifact_repository.rename_async("new_name")
        assert isinstance(op, PollingOperation)
        op.result()
        mock_finalizer.assert_called_once()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_rename_artifact_repository_with_options(fake_root, artifact_repository):
    args = (
        fake_root,
        "POST",
        BASE_URL
        + "/databases/my_db/schemas/my_schema/artifact-repositories/my_artifact_repository:rename?"
        + "ifExists=True&targetDatabase=other_db&targetSchema=other_schema&targetName=new_name",
    )
    kwargs = extra_params(
        query_params=[
            ("ifExists", True),
            ("targetDatabase", "other_db"),
            ("targetSchema", "other_schema"),
            ("targetName", "new_name"),
        ]
    )

    with (
        mock.patch(API_CLIENT_REQUEST) as mocked_request,
        mock.patch.object(artifact_repository, "_rename_finalizer") as mock_finalizer,
    ):
        artifact_repository.rename("new_name", if_exists=True, target_database="other_db", target_schema="other_schema")
        mock_finalizer.assert_called_once()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_create_with_mode_or_replace(fake_root, artifact_repositories):
    from snowflake.core._common import CreateMode

    args = (
        fake_root,
        "POST",
        BASE_URL + "/databases/my_db/schemas/my_schema/artifact-repositories?createMode=orReplace",
    )
    kwargs = extra_params(
        query_params=[("createMode", "orReplace")],
        body={"name": "my_artifact_repository", "type": "PIP", "api_integration": "my_api_integration"},
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        artifact_repositories.create(ARTIFACT_REPOSITORY, mode=CreateMode.or_replace)
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_create_with_mode_if_not_exists(fake_root, artifact_repositories):
    from snowflake.core._common import CreateMode

    args = (
        fake_root,
        "POST",
        BASE_URL + "/databases/my_db/schemas/my_schema/artifact-repositories?createMode=ifNotExists",
    )
    kwargs = extra_params(
        query_params=[("createMode", "ifNotExists")],
        body={"name": "my_artifact_repository", "type": "PIP", "api_integration": "my_api_integration"},
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        artifact_repositories.create(ARTIFACT_REPOSITORY, mode=CreateMode.if_not_exists)
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_create_with_string_mode(fake_root, artifact_repositories):
    args = (
        fake_root,
        "POST",
        BASE_URL + "/databases/my_db/schemas/my_schema/artifact-repositories?createMode=orReplace",
    )
    kwargs = extra_params(
        query_params=[("createMode", "orReplace")],
        body={"name": "my_artifact_repository", "type": "PIP", "api_integration": "my_api_integration"},
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        artifact_repositories.create(ARTIFACT_REPOSITORY, mode="or_replace")
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_with_multiple_filters(fake_root, artifact_repositories):
    """Test iter with multiple filter parameters combined."""
    args = (
        fake_root,
        "GET",
        BASE_URL
        + "/databases/my_db/schemas/my_schema/artifact-repositories?"
        + "like=test%&startsWith=test&showLimit=50&fromName=start_repo",
    )
    kwargs = extra_params(
        query_params=[
            ("like", "test%"),
            ("startsWith", "test"),
            ("showLimit", 50),
            ("fromName", "start_repo"),
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        list(artifact_repositories.iter(like="test%", starts_with="test", limit=50, from_name="start_repo"))
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_create_or_alter_name_mismatch(artifact_repositories):
    """Test that create_or_alter raises ValueError when names don't match."""
    mismatched_repo = ArtifactRepository(name="different_name", type="PIP", api_integration="my_api_integration")

    with pytest.raises(ValueError, match="Cannot call create_or_alter on a resource with a different name"):
        artifact_repositories["my_artifact_repository"].create_or_alter(mismatched_repo)


def test_resource_string_representation(artifact_repositories):
    """Test string representation of resources."""
    artifact_repo = artifact_repositories["my_artifact_repository"]
    assert "my_artifact_repository" in str(artifact_repo)
    assert "ArtifactRepositoryResource" in str(artifact_repo)


def test_collection_indexing(artifact_repositories):
    """Test collection indexing and item access."""
    # Test __getitem__
    repo = artifact_repositories["test_repo"]
    assert repo.name == "test_repo"

    # Test that different names create different resource instances
    repo1 = artifact_repositories["repo1"]
    repo2 = artifact_repositories["repo2"]
    assert repo1.name != repo2.name


def test_create_or_alter_new_resource(fake_root, artifact_repositories):
    """Test create_or_alter on a non-existing resource (creation scenario)."""
    new_repo = ArtifactRepository(
        name="brand_new_repository",
        type="PIP",
        api_integration="my_api_integration",
        comment="Created via create_or_alter",
    )

    args = (
        fake_root,
        "PUT",
        BASE_URL + "/databases/my_db/schemas/my_schema/artifact-repositories/brand_new_repository",
    )
    kwargs = extra_params(
        body={
            "name": "brand_new_repository",
            "type": "PIP",
            "api_integration": "my_api_integration",
            "comment": "Created via create_or_alter",
        },
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        result = artifact_repositories["brand_new_repository"].create_or_alter(new_repo)
        assert result is None  # create_or_alter returns None for both create and update
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_invalid_repository_type():
    """Test that invalid repository type raises validation error."""
    with pytest.raises(ValueError, match="must validate the enum values"):
        ArtifactRepository(
            name="test_repo",
            type="INVALID_TYPE",  # Invalid type - only PIP is supported
            api_integration="test_api_integration",
        )


def test_invalid_repository_name():
    """Test that invalid repository name raises validation error."""
    with pytest.raises(ValueError, match="must validate the regular expression"):
        ArtifactRepository(
            name="invalid-name-with-hyphens!",  # Invalid characters
            type="PIP",
            api_integration="test_api_integration",
        )



"""
Snowflake Stage API.

The Snowflake Stage API is a REST API that you can use to access, update, and perform certain actions on stage resources in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Optional

from pydantic import BaseModel, ConfigDict, SecretStr, StrictStr, field_validator


class StageEncryption(BaseModel):
    """A model object representing the StageEncryption resource.

    Constructs an object of type StageEncryption with the provided properties.

    Parameters
    __________
    type : str, optional
        Specifies the encryption type used.
    master_key : str, optional
        Specifies the client-side master key used to encrypt the files in the bucket. The master key must be a 128-bit or 256-bit key in Base64-encoded form.
    kms_key_id : str, optional
        Optionally specifies the ID for the KMS-managed key used to encrypt files unloaded into the bucket.
    """

    type: Optional[StrictStr] = None

    master_key: Optional[SecretStr] = None

    kms_key_id: Optional[StrictStr] = None

    __properties = ["type", "master_key", "kms_key_id"]

    @field_validator("type")
    def type_validate_enum(cls, v):
        if v is None:
            return v
        if v not in (
            "SNOWFLAKE_FULL",
            "SNOWFLAKE_SSE",
            "AWS_CSE",
            "AWS_SSE_S3",
            "AWS_SSE_KMS",
            "GCS_SSE_KMS",
            "AZURE_CSE",
            "NONE",
        ):
            raise ValueError(
                "must validate the enum values ('SNOWFLAKE_FULL','SNOWFLAKE_SSE','AWS_CSE','AWS_SSE_S3','AWS_SSE_KMS','GCS_SSE_KMS','AZURE_CSE','NONE')"
            )
        return v

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> StageEncryption:
        """Create an instance of StageEncryption from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> StageEncryption:
        """Create an instance of StageEncryption from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return StageEncryption.model_validate(obj)

        _obj = StageEncryption.model_validate(
            {
                "type": obj.get("type"),
                "master_key": obj.get("master_key"),
                "kms_key_id": obj.get("kms_key_id"),
            }
        )

        return _obj


class StageEncryptionModel:
    def __init__(
        self,  # optional properties
        type: Optional[str] = None,
        master_key: Optional[str] = None,
        kms_key_id: Optional[str] = None,
    ):
        """A model object representing the StageEncryption resource.

        Constructs an object of type StageEncryption with the provided properties.

        Parameters
        __________
        type : str, optional
            Specifies the encryption type used.
        master_key : str, optional
            Specifies the client-side master key used to encrypt the files in the bucket. The master key must be a 128-bit or 256-bit key in Base64-encoded form.
        kms_key_id : str, optional
            Optionally specifies the ID for the KMS-managed key used to encrypt files unloaded into the bucket.
        """
        self.type = type
        self.master_key = master_key
        self.kms_key_id = kms_key_id

    __properties = ["type", "master_key", "kms_key_id"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return StageEncryption(
            type=self.type,
            master_key=self.master_key,
            kms_key_id=self.kms_key_id,
        )

    @classmethod
    def _from_model(cls, model) -> StageEncryptionModel:
        return StageEncryptionModel(
            type=model.type,
            master_key=model.master_key,
            kms_key_id=model.kms_key_id,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a StageEncryption.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the StageEncryption object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> StageEncryptionModel:
        """Create an instance of StageEncryption from a dict.

        This method constructs a StageEncryption object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        StageEncryption
            A StageEncryption object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(StageEncryption.from_dict(obj))


StageEncryption._model_class = StageEncryptionModel



import copy

import pytest as pytest

from snowflake.core.table import Table, TableColumn
from snowflake.core.view import View, ViewColumn
from tests.integ.utils import random_string


temp_table_template = Table(
    name="ToBeSet",
    columns=[
        TableColumn(
            name="c1",
            datatype="int",
            nullable=False,
            autoincrement=True,
            autoincrement_start=0,
            autoincrement_increment=1,
        ),
        TableColumn(name="c2", datatype="string"),
    ],
)


@pytest.fixture(scope="module")
def src_temp_table(tables):
    table_name = random_string(10, "stream_src_table_")
    table_template = copy.deepcopy(temp_table_template)
    table_template.name = table_name
    table_handle = tables.create(table_template)
    try:
        yield table_handle
    finally:
        tables[table_name].drop()


@pytest.fixture(scope="module")
def src_temp_view(views, src_temp_table):
    view_name = random_string(10, "stream_src_view_")
    view_handle = views.create(
        View(
            name=view_name,
            columns=[ViewColumn(name="c1"), ViewColumn(name="c2")],
            query=f"SELECT * FROM {src_temp_table.name}",
        )
    )
    try:
        yield view_handle
    finally:
        views[view_name].drop()


@pytest.fixture(scope="module")
def src_temp_view_with_multiple_base_tables(views, tables, src_temp_table):
    view_name = random_string(10, "stream_src_view_")
    other_table_name = random_string(10, "stream_src_other_table_")

    table_template = copy.deepcopy(temp_table_template)
    table_template.name = other_table_name
    table_handle = tables.create(table_template)

    view_handle = views.create(
        View(
            name=view_name,
            columns=[ViewColumn(name="c1"), ViewColumn(name="c2")],
            query=f"""\
            SELECT
                {src_temp_table.name}.c1,
                {table_handle.name}.c2
            FROM {src_temp_table.name}
            JOIN {table_handle.name}
            ON {src_temp_table.name}.c2 = {table_handle.name}.c1
        """,
        )
    )

    try:
        yield view_handle, src_temp_table.name, table_handle.name
    finally:
        views[view_name].drop()
        tables[other_table_name].drop()



"""
Cortex Inference API.

OpenAPI 3.0 specification for the Cortex REST API  # noqa: E501

The version of the OpenAPI document: 0.1.0
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, List, Optional

from pydantic import BaseModel, ConfigDict, StrictStr, field_validator

from snowflake.core.cortex.inference_service._generated.models.non_streaming_complete_response_choices_inner import (
    NonStreamingCompleteResponseChoicesInner,
    NonStreamingCompleteResponseChoicesInnerModel,
)
from snowflake.core.cortex.inference_service._generated.models.usage import Usage, UsageModel


class NonStreamingCompleteResponse(BaseModel):
    """A model object representing the NonStreamingCompleteResponse resource.

    Constructs an object of type NonStreamingCompleteResponse with the provided properties.

    Parameters
    __________
    choices : list[NonStreamingCompleteResponseChoicesInner], optional

    finish_reason : str, optional
        The reason the request was finished.
    usage : Usage, optional
    """

    choices: Optional[List[NonStreamingCompleteResponseChoicesInner]] = None

    finish_reason: Optional[StrictStr] = None

    usage: Optional[Usage] = None

    __properties = ["choices", "finish_reason", "usage"]

    @field_validator("finish_reason")
    def finish_reason_validate_enum(cls, v):
        if v is None:
            return v
        if v not in ("stop", "content_filter", "tool_calls", "length"):
            raise ValueError("must validate the enum values ('stop','content_filter','tool_calls','length')")
        return v

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> NonStreamingCompleteResponse:
        """Create an instance of NonStreamingCompleteResponse from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of each item in choices (list)
        _items = []
        if self.choices:
            for _item in self.choices:
                if _item:
                    _items.append(_item.to_dict())
            _dict["choices"] = _items

        # override the default output from pydantic by calling `to_dict()` of usage
        if self.usage:
            _dict["usage"] = self.usage.to_dict()

        # set to None if finish_reason (nullable) is None
        if self.finish_reason is None:
            _dict["finish_reason"] = None

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> NonStreamingCompleteResponse:
        """Create an instance of NonStreamingCompleteResponse from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return NonStreamingCompleteResponse.model_validate(obj)

        _obj = NonStreamingCompleteResponse.model_validate(
            {
                "choices": [NonStreamingCompleteResponseChoicesInner.from_dict(_item) for _item in obj.get("choices")]
                if obj.get("choices") is not None
                else None,
                "finish_reason": obj.get("finish_reason"),
                "usage": Usage.from_dict(obj.get("usage")) if obj.get("usage") is not None else None,
            }
        )

        return _obj


class NonStreamingCompleteResponseModel:
    def __init__(
        self,  # optional properties
        choices: Optional[list[NonStreamingCompleteResponseChoicesInner]] = None,
        finish_reason: Optional[str] = None,
        usage: Optional[Usage] = None,
    ):
        """A model object representing the NonStreamingCompleteResponse resource.

        Constructs an object of type NonStreamingCompleteResponse with the provided properties.

        Parameters
        __________
        choices : list[NonStreamingCompleteResponseChoicesInner], optional

        finish_reason : str, optional
            The reason the request was finished.
        usage : Usage, optional
        """
        self.choices = choices
        self.finish_reason = finish_reason
        self.usage = usage

    __properties = ["choices", "finish_reason", "usage"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return NonStreamingCompleteResponse(
            choices=[x._to_model() for x in self.choices] if self.choices is not None else None,
            finish_reason=self.finish_reason,
            usage=self.usage._to_model() if self.usage is not None else None,
        )

    @classmethod
    def _from_model(cls, model) -> NonStreamingCompleteResponseModel:
        return NonStreamingCompleteResponseModel(
            choices=[NonStreamingCompleteResponseChoicesInnerModel._from_model(x) for x in model.choices]
            if model.choices
            else None,
            finish_reason=model.finish_reason,
            usage=UsageModel._from_model(model.usage) if model.usage else None,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a NonStreamingCompleteResponse.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the NonStreamingCompleteResponse object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> NonStreamingCompleteResponseModel:
        """Create an instance of NonStreamingCompleteResponse from a dict.

        This method constructs a NonStreamingCompleteResponse object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        NonStreamingCompleteResponse
            A NonStreamingCompleteResponse object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(NonStreamingCompleteResponse.from_dict(obj))


NonStreamingCompleteResponse._model_class = NonStreamingCompleteResponseModel



import pytest

from snowflake.core._common import CreateMode
from snowflake.core.exceptions import NotFoundError
from snowflake.core.warehouse import Warehouse

from ..utils import random_string


pytestmark = pytest.mark.usefixtures("backup_warehouse_fixture")


def test_drop_warehouse(warehouses):
    warehouse_name = random_string(5, "test_create_warehouse_")
    test_warehouse = Warehouse(name=warehouse_name, warehouse_size="SMALL", auto_suspend=500)

    warehouse_ref = None
    try:
        warehouse_ref = warehouses.create(test_warehouse, mode=CreateMode.error_if_exists)
        warehouse = warehouse_ref.fetch()
        assert warehouse_name.upper() == warehouse.name.upper()
        warehouse_ref.drop()

        with pytest.raises(NotFoundError):
            warehouse_ref.fetch()

        with pytest.raises(NotFoundError):
            warehouse_ref.drop()

        # Should not error
        warehouse_ref.drop(if_exists=True)

        warehouse_ref = warehouses.create(test_warehouse, mode=CreateMode.error_if_exists)
        warehouse = warehouse_ref.fetch()
        assert warehouse_name.upper() == warehouse.name.upper()
    finally:
        warehouse_ref.drop()



# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.

import platform
import sys


SESSION_TOKEN_EXPIRED_ERROR_CODE = "390112"

# Don't change the constant after this point
PYTHON_VERSION = ".".join(str(v) for v in sys.version_info[:3])
PLATFORM = platform.platform()



"""
Cortex Analyst API.

The Snowflake Cortex Analyst API is a REST API that allows end user to chat with their data leveraging semantic models to generate SQL queries.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any

from pydantic import BaseModel, ConfigDict, StrictStr


class TableauAuth(BaseModel):
    """A model object representing the TableauAuth resource.

    Constructs an object of type TableauAuth with the provided properties.

    Parameters
    __________
    secret_fqn : str
        The fully qualified name of the secret containing the Tableau PAT
    server_url : str
        The Tableau server name to send requests to
    """

    secret_fqn: StrictStr

    server_url: StrictStr

    __properties = ["secret_fqn", "server_url"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> TableauAuth:
        """Create an instance of TableauAuth from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> TableauAuth:
        """Create an instance of TableauAuth from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return TableauAuth.model_validate(obj)

        _obj = TableauAuth.model_validate(
            {
                "secret_fqn": obj.get("secret_fqn"),
                "server_url": obj.get("server_url"),
            }
        )

        return _obj


class TableauAuthModel:
    def __init__(
        self,
        secret_fqn: str,
        server_url: str,
        # optional properties
    ):
        """A model object representing the TableauAuth resource.

        Constructs an object of type TableauAuth with the provided properties.

        Parameters
        __________
        secret_fqn : str
            The fully qualified name of the secret containing the Tableau PAT
        server_url : str
            The Tableau server name to send requests to
        """
        self.secret_fqn = secret_fqn
        self.server_url = server_url

    __properties = ["secret_fqn", "server_url"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return TableauAuth(
            secret_fqn=self.secret_fqn,
            server_url=self.server_url,
        )

    @classmethod
    def _from_model(cls, model) -> TableauAuthModel:
        return TableauAuthModel(
            secret_fqn=model.secret_fqn,
            server_url=model.server_url,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a TableauAuth.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the TableauAuth object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> TableauAuthModel:
        """Create an instance of TableauAuth from a dict.

        This method constructs a TableauAuth object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        TableauAuth
            A TableauAuth object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(TableauAuth.from_dict(obj))


TableauAuth._model_class = TableauAuthModel



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from snowflake.core.event_table._generated.api.event_table_api import EventTableApi

__all__ = [
    "EventTableApi",
]



from snowflake.core.grant import Grant, Grantee, Privileges, Securable


def test_to_dict():
    grant = Grant(
        grantee=Grantee(name="test_role", grantee_type="foo"),
        securable=Securable(name="test_securable", securable_type="bar"),
        privileges=[Privileges.add_search_optimization, Privileges.read],
        grant_option=False,
    )
    assert grant.to_dict() == {
        "grant_option": False,
        "privileges": ["ADD SEARCH OPTIMIZATION", "READ"],
        "securable_type": "bar",
    }



from snowflake.core.image_repository import ImageRepository


def test_fetch(image_repositories, temp_ir):
    ir: ImageRepository = image_repositories[temp_ir.name].fetch()
    assert ir.name == temp_ir.name.upper()  # for upper/lower case names
    assert ir.created_on
    assert ir.repository_url
    assert ir.owner
    assert ir.owner_role_type



#
# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.
#


import pytest

from snowflake.core.database import Database, DatabaseCollection

from ..utils import random_string


pytestmark = pytest.mark.usefixtures("backup_database_schema")


def test_resist_multi_statement_sql_injection(databases: DatabaseCollection):
    new_db_name = random_string(3, "test_db_resist_multi_statement_sql_injection_")
    sql_injection_comment = "'comment for disguise'; select '1'"

    new_db = Database(name=new_db_name, comment=sql_injection_comment)

    db = databases.create(new_db)
    try:
        db.fetch()
    finally:
        db.drop()



import pytest

from snowflake.core.artifact_repository import ArtifactRepository
from snowflake.core.exceptions import NotFoundError
from tests.integ.utils import random_string


def test_drop_artifact_repository(artifact_repositories, temp_pypi_api_integration):
    """Test dropping an artifact repository."""
    ar_name = random_string(10, "test_artifact_repository_drop_")
    ar_def = ArtifactRepository(
        name=ar_name,
        type="PIP",
        api_integration=temp_pypi_api_integration,
        comment="created by test_drop_artifact_repository",
    )
    ar = artifact_repositories.create(ar_def)

    fetched_ar = ar.fetch()
    assert fetched_ar.name == ar_name.upper()

    ar.drop()

    with pytest.raises(NotFoundError):
        ar.fetch()


def test_drop_artifact_repository_if_exists(artifact_repositories, temp_pypi_api_integration):
    """Test dropping an artifact repository with if_exists=True."""
    ar_name = random_string(10, "test_artifact_repository_drop_if_exists_")
    ar_def = ArtifactRepository(
        name=ar_name,
        type="PIP",
        api_integration=temp_pypi_api_integration,
        comment="created by test_drop_artifact_repository_if_exists",
    )
    ar = artifact_repositories.create(ar_def)

    ar.drop(if_exists=True)

    with pytest.raises(NotFoundError):
        ar.fetch()


def test_drop_nonexistent_artifact_repository_if_exists(artifact_repositories):
    """Test dropping a non-existent artifact repository with if_exists=True."""
    ar_name = random_string(10, "test_artifact_repository_nonexistent_")
    ar = artifact_repositories[ar_name]

    ar.drop(if_exists=True)


def test_drop_nonexistent_artifact_repository_without_if_exists(artifact_repositories):
    """Test dropping a non-existent artifact repository without if_exists."""
    ar_name = random_string(10, "test_artifact_repository_nonexistent_no_if_exists_")
    ar = artifact_repositories[ar_name]

    with pytest.raises(NotFoundError):
        ar.drop()



"""
Snowflake API Integration API.

The Snowflake API Integration API is a REST API that you can use to access, update, and perform certain actions on API Integration resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re

from typing import Any, Optional

from pydantic import BaseModel, ConfigDict, Field, field_validator
from typing_extensions import Annotated


class TagReference(BaseModel):
    """A model object representing the TagReference resource.

    Constructs an object of type TagReference with the provided properties.

    Parameters
    __________
    tag_name : str
        The name of the tag.
    tag_database : str, optional
        The database in which the tag is defined.
    tag_schema : str, optional
        The schema in which the tag is defined.
    """

    tag_database: Optional[Annotated[str, Field(strict=True)]] = None

    tag_schema: Optional[Annotated[str, Field(strict=True)]] = None

    tag_name: Annotated[str, Field(strict=True)]

    __properties = ["tag_database", "tag_schema", "tag_name"]

    @field_validator("tag_database")
    def tag_database_validate_regular_expression(cls, v):
        if v is None:
            return v
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    @field_validator("tag_schema")
    def tag_schema_validate_regular_expression(cls, v):
        if v is None:
            return v
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    @field_validator("tag_name")
    def tag_name_validate_regular_expression(cls, v):
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> TagReference:
        """Create an instance of TagReference from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> TagReference:
        """Create an instance of TagReference from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return TagReference.model_validate(obj)

        _obj = TagReference.model_validate(
            {
                "tag_database": obj.get("tag_database"),
                "tag_schema": obj.get("tag_schema"),
                "tag_name": obj.get("tag_name"),
            }
        )

        return _obj


class TagReferenceModel:
    def __init__(
        self,
        tag_name: str,
        # optional properties
        tag_database: Optional[str] = None,
        tag_schema: Optional[str] = None,
    ):
        """A model object representing the TagReference resource.

        Constructs an object of type TagReference with the provided properties.

        Parameters
        __________
        tag_name : str
            The name of the tag.
        tag_database : str, optional
            The database in which the tag is defined.
        tag_schema : str, optional
            The schema in which the tag is defined.
        """
        self.tag_database = tag_database
        self.tag_schema = tag_schema
        self.tag_name = tag_name

    __properties = ["tag_database", "tag_schema", "tag_name"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return TagReference(
            tag_database=self.tag_database,
            tag_schema=self.tag_schema,
            tag_name=self.tag_name,
        )

    @classmethod
    def _from_model(cls, model) -> TagReferenceModel:
        return TagReferenceModel(
            tag_database=model.tag_database,
            tag_schema=model.tag_schema,
            tag_name=model.tag_name,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a TagReference.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the TagReference object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> TagReferenceModel:
        """Create an instance of TagReference from a dict.

        This method constructs a TagReference object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        TagReference
            A TagReference object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(TagReference.from_dict(obj))


TagReference._model_class = TagReferenceModel



from contextlib import suppress

import pytest as pytest

from snowflake.core.exceptions import NotFoundError
from tests.integ.utils import random_string


@pytest.mark.min_sf_ver("8.34.0")
@pytest.mark.use_accountadmin
def test_iter(network_policies, template_network_policy, prepare_rules_for_network_polices):
    names_list = []
    prefix = random_string(5, "test_network_policies_iter_").upper()
    for _ in range(5):
        names_list.append(random_string(10, prefix).upper())

    allowed_rules, blocked_rules = prepare_rules_for_network_polices

    try:
        for name in names_list:
            np = template_network_policy
            np.name = name
            network_policies.create(np)

        tests_network_policies = [np for np in network_policies.iter() if np.name.startswith(prefix)]

        assert len(tests_network_policies) >= 5

        for np in tests_network_policies:
            assert np.name in names_list
            assert allowed_rules == np.allowed_network_rule_list
            assert blocked_rules == np.blocked_network_rule_list
            assert ["8.8.8.8"] == np.allowed_ip_list
            assert ["0.0.0.0"] == np.blocked_ip_list

    finally:
        for name in names_list:
            with suppress(NotFoundError):
                network_policies[name].drop()



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from snowflake.core.network_policy._generated.api.network_policy_api import NetworkPolicyApi

__all__ = [
    "NetworkPolicyApi",
]



#
# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.
#


import pytest


pytestmark = pytest.mark.usefixtures("backup_database_schema")


@pytest.mark.min_sf_ver("9.8.0")
def test_fetch(databases, temp_db):
    database = databases[temp_db.name].fetch()
    assert database.name.upper() == temp_db.name.upper()
    assert database.comment == "created by temp_db"
    assert database.serverless_task_min_statement_size == "XSMALL"
    assert database.serverless_task_max_statement_size == "X2LARGE"
    assert database.user_task_managed_initial_warehouse_size == "MEDIUM"
    assert database.suspend_task_after_num_failures == 10
    assert database.user_task_timeout_ms == 3600000



import logging

import pytest

from snowflake.core.exceptions import NotFoundError


def test_request_id_logging_negative(root, caplog):
    with caplog.at_level(logging.DEBUG, "snowflake.core"):
        with pytest.raises(NotFoundError) as e:
            root.databases["nonexistent_db"].fetch()
        request_id = e.value.request_id
    assert any(request_id in m for m in caplog.messages)



"""
Cortex Search REST API.

OpenAPI 3.0 specification for the Cortex Search REST API  # noqa: E501

The version of the OpenAPI document: 0.1.0
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, List

from pydantic import BaseModel, ConfigDict, StrictStr

from snowflake.core.cortex.search_service._generated.models.suggest_result import SuggestResult, SuggestResultModel


class SuggestQueryResponse(BaseModel):
    """A model object representing the SuggestQueryResponse resource.

    Constructs an object of type SuggestQueryResponse with the provided properties.

    Parameters
    __________
    results : list[SuggestResult]
        List of result rows.
    request_id : str
        ID of the request.
    """

    results: List[SuggestResult]

    request_id: StrictStr

    __properties = ["results", "request_id"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> SuggestQueryResponse:
        """Create an instance of SuggestQueryResponse from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of each item in results (list)
        _items = []
        if self.results:
            for _item in self.results:
                if _item:
                    _items.append(_item.to_dict())
            _dict["results"] = _items

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> SuggestQueryResponse:
        """Create an instance of SuggestQueryResponse from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return SuggestQueryResponse.model_validate(obj)

        _obj = SuggestQueryResponse.model_validate(
            {
                "results": [SuggestResult.from_dict(_item) for _item in obj.get("results")]
                if obj.get("results") is not None
                else None,
                "request_id": obj.get("request_id"),
            }
        )

        return _obj


class SuggestQueryResponseModel:
    def __init__(
        self,
        results: list[SuggestResult],
        request_id: str,
        # optional properties
    ):
        """A model object representing the SuggestQueryResponse resource.

        Constructs an object of type SuggestQueryResponse with the provided properties.

        Parameters
        __________
        results : list[SuggestResult]
            List of result rows.
        request_id : str
            ID of the request.
        """
        self.results = results
        self.request_id = request_id

    __properties = ["results", "request_id"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return SuggestQueryResponse(
            results=[x._to_model() for x in self.results] if self.results is not None else None,
            request_id=self.request_id,
        )

    @classmethod
    def _from_model(cls, model) -> SuggestQueryResponseModel:
        return SuggestQueryResponseModel(
            results=[SuggestResultModel._from_model(x) for x in model.results] if model.results else None,
            request_id=model.request_id,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a SuggestQueryResponse.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the SuggestQueryResponse object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> SuggestQueryResponseModel:
        """Create an instance of SuggestQueryResponse from a dict.

        This method constructs a SuggestQueryResponse object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        SuggestQueryResponse
            A SuggestQueryResponse object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(SuggestQueryResponse.from_dict(obj))


SuggestQueryResponse._model_class = SuggestQueryResponseModel



from ._embed_service import CortexEmbedService, EmbedRequest
from ._generated import CortexEmbedApi


CortexEmbedServiceApi = CortexEmbedApi

__all__ = ["CortexEmbedService", "EmbedRequest", "CortexEmbedServiceApi"]



import pytest

from snowflake.core._common import CreateMode
from snowflake.core.event_table import EventTable
from snowflake.core.exceptions import ConflictError, NotFoundError
from tests.utils import random_string


@pytest.mark.min_sf_ver("8.35.0")
def test_create_and_fetch(event_tables):
    event_table_name = random_string(10, "test_create_event_table_")

    try:
        event_table_created = event_tables.create(
            EventTable(
                name=event_table_name,
                data_retention_time_in_days=1,
                max_data_extension_time_in_days=1,
                change_tracking=True,
                default_ddl_collation="EN-CI",
                comment="ThIs (*)@#$ is CoM%t",
            )
        )

        event_table_handle = event_table_created.fetch()
        assert event_table_handle.name.upper() == event_table_name.upper()
        assert event_table_handle.comment == "ThIs (*)@#$ is CoM%t"
        assert event_table_handle.data_retention_time_in_days == 1
        assert event_table_handle.max_data_extension_time_in_days == 1
        assert event_table_handle.change_tracking is True
        assert event_table_handle.default_ddl_collation == "EN-CI"

        # check the basic fields are present and not none
        assert event_table_handle.created_on is not None
        assert event_table_handle.owner is not None
        assert event_table_handle.owner_role_type is not None
        assert event_table_handle.rows is not None
        assert event_table_handle.bytes is not None
        assert event_table_handle.automatic_clustering is not None
        assert event_table_handle.search_optimization is not None
        assert event_table_handle.columns is not None

        created_time = event_table_handle.created_on

        with pytest.raises(ConflictError):
            event_tables.create(EventTable(name=event_table_name))

        assert (
            created_time
            == event_tables.create(EventTable(name=event_table_name), mode=CreateMode.if_not_exists).fetch().created_on
        )

        # check replace with fetched result
        event_table_handle.comment = "This is a new comment"
        event_table_created_new = event_tables.create(event_table_handle, mode=CreateMode.or_replace)
        event_table_created_new_handle = event_table_created_new.fetch()
        assert event_table_created_new_handle.comment == "This is a new comment"

        with pytest.raises(NotFoundError):
            event_tables["dummy"].fetch()

    finally:
        event_tables[event_table_name].drop(if_exists=True)



"""
Cortex Analyst API.

The Snowflake Cortex Analyst API is a REST API that allows end user to chat with their data leveraging semantic models to generate SQL queries.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from enum import Enum


class SemanticModelChangeOperation(str, Enum):
    """NOTE: This class is auto generated by OpenAPI Generator.

    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    allowed enum values
    """


SemanticModelChangeOperationModel = SemanticModelChangeOperation



from dataclasses import dataclass
from typing import TYPE_CHECKING, Optional

from ._generated.api.tag_api_base import TagCollectionBase, TagResourceBase


if TYPE_CHECKING:
    from snowflake.core.schema import SchemaResource


class TagCollection(TagCollectionBase):
    """Represents the collection operations on the Snowflake tag resource.

    With this collection, you can create, iterate through, and search for tags that you have access to
    in the current context.
    """

    def __init__(self, schema: "SchemaResource"):
        super().__init__(schema, TagResource)


class TagResource(TagResourceBase):
    """Represents a reference to a Snowflake tag.

    With this tag reference, you can fetch information about a tag, as well as
    perform certain actions on it.
    """

    _plural_name = "tags"


@dataclass
class TagValue:
    """Represents a value of a Snowflake tag.

    With this object, you can set tags on resources. It is also used when fetching tags.

    Parameters
    __________
    value: str
    level: bool, optional. Read-only; used only when fetching tags and ignored when provided in set-tags call.
    """

    value: str
    level: Optional[str] = None



import copy

import pytest

from snowflake.core.exceptions import NotFoundError
from tests.integ.utils import random_string

from .conftest import test_generic_string_secret_template


def test_drop(secrets):
    prefix = "test_secret_drop_"
    pre_create_count = len(list(secrets.iter(like=prefix + "%")))

    name = random_string(10, prefix)
    secret_handle = secrets[name]

    secret = copy.deepcopy(test_generic_string_secret_template)
    secret.name = name
    secrets.create(secret)

    created_count = len(list(secrets.iter(like=prefix + "%")))
    assert pre_create_count + 1 == created_count

    secret_handle.drop()

    after_drop_count = len(list(secrets.iter(like=prefix + "%")))
    assert pre_create_count == after_drop_count

    with pytest.raises(NotFoundError):
        secret_handle.drop()
        secret_handle.drop(if_exists=False)

    secret_handle.drop(if_exists=True)



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from snowflake.core.spark_connect._generated.api.spark_connect_api import SparkConnectApi

__all__ = [
    "SparkConnectApi",
]



from collections import defaultdict
from contextlib import suppress
from unittest import mock

from snowflake.core.database import Database
from snowflake.core.version import __version__ as VERSION


SNOWPY_USER_AGENT_VAL = "python_api/" + VERSION


def test_fetch_with_ua(fake_root, db):
    fake_root.root_config.get_user_agents = mock.MagicMock(return_value="customized_ua snowFlake/412.123.432a")
    fake_root.root_config.has_user_agents = mock.MagicMock(return_value=True)
    with suppress(Exception):
        with mock.patch("snowflake.core._generated.api_client.ApiClient.request") as mocked_request:
            db.fetch()
    try:
        mocked_request.assert_called_once_with(
            fake_root,
            "GET",
            "http://localhost:80/api/v2/databases/my_db",
            query_params=[],
            headers={
                "Accept": "application/json",
                "User-Agent": SNOWPY_USER_AGENT_VAL + " customized_ua snowFlake/412.123.432a",
            },
            post_params=[],
            body=None,
            _preload_content=True,
            _request_timeout=None,
        )
    finally:
        fake_root.root_config.get_user_agents = mock.MagicMock(return_value="")
        fake_root.root_config.has_user_agents = mock.MagicMock(return_value=False)


def test_create_from_share(fake_root, dbs):
    with mock.patch("snowflake.core._generated.api_client.ApiClient.request") as mocked_request:
        dbs.create(database=Database(name="name"), from_share="share")
    mocked_request.assert_called_once_with(
        fake_root,
        "POST",
        "http://localhost:80/api/v2/databases:from-share?createMode=errorIfExists&share=share",
        query_params=[("createMode", "errorIfExists"), ("share", "share")],
        headers={"Accept": "application/json", "Content-Type": "application/json", "User-Agent": SNOWPY_USER_AGENT_VAL},
        post_params=[],
        body={"name": "name"},
        _preload_content=True,
        _request_timeout=None,
    )


request_called_count_for_long_running = defaultdict(int)


def mocked_request_get(*args, **kwargs):
    class MockResponse:
        def __init__(self, data, status):
            self.data = data
            self.status = status

        def getheader(self, param):
            return {"Location": "/api/v2/results/XXXX"}.get(param, None)

    url = args[2]

    global request_called_count_for_long_running
    if "api/v2/databases" in url:
        request_called_count_for_long_running["database"] += 1
        return MockResponse(None, 202)

    assert "api/v2/result" in url
    request_called_count_for_long_running["result"] += 1
    if request_called_count_for_long_running.get("result", 0) < 2:
        return MockResponse(None, 202)
    return None


# def test_fetch_with_long_running(fake_root, setup_enable_rest_api_with_long_running):
#     dbs = DatabaseCollection(fake_root)
#     with setup_enable_rest_api_with_long_running(dbs._ref_class):
#         with mock.patch(
#             "snowflake.core.database._generated.api_client.ApiClient.request",
#             side_effect=mocked_request_get,
#         ):
#             # TODO: Improve fetch results instead of error
#             try:
#                 dbs["my_db"].fetch()
#             except AttributeError:
#                 # TODO: The AttributeError message is not reliable, so we just catch it and
#                 # pass for now. Make this better
#                 # assert e.args[0] == "'NoneType' object has no attribute 'status'"
#                 pass
#         global request_called_count_for_long_running
#         assert request_called_count_for_long_running['database'] == 1
#         assert request_called_count_for_long_running['result'] == 2


def test_fetch_with_throttling(fake_root, db):
    class MockResponse:
        headers = {"Location": "/api/v2/databases"}

        def __init__(self, data, status, reason):
            self.data = data
            self.status = status
            self.reason = reason

    with mock.patch("snowflake.core._http_requests.SFPoolManager.request") as mocked_request:
        # Triple Retry into Success
        http_throttle_codes = [429, 503, 504, 200]

        def side_effect(*args, **kwargs):
            status_code = http_throttle_codes.pop(0)
            return MockResponse('{"name": "name"}', status_code, "")

        mocked_request.side_effect = side_effect
        db.fetch()
    assert mocked_request.call_count == 4
    mocked_request.assert_called_with(
        fake_root,
        "GET",
        "http://localhost:80/api/v2/databases/my_db",
        fields={},
        preload_content=True,
        timeout=mock.ANY,
        headers={"Accept": "application/json", "User-Agent": SNOWPY_USER_AGENT_VAL},
    )



"""
Snowflake Account API.

The Snowflake Account API is a REST API that you can use to access, update, and perform certain actions on Account resource in Snowflake.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

import json
import logging
import os
import re
import tempfile

from concurrent.futures import Future
from datetime import date, datetime
from typing import Iterable, Literal, Optional, Union, overload

from dateutil.parser import parse
from pydantic import Field, StrictBool, StrictInt, StrictStr, validate_call
from typing_extensions import Annotated

import snowflake.core._http_requests

from snowflake.core._generated import rest
from snowflake.core._generated.api_client import ApiClient
from snowflake.core._internal.utils import is_running_inside_stored_procedure
from snowflake.core.account._generated import models
from snowflake.core.account._generated.models.account import Account
from snowflake.core.account._generated.models.success_response import SuccessResponse
from snowflake.core.exceptions import (  # noqa: F401
    _APITypeError,
    _APIValueError,
)


logger = logging.getLogger(__name__)


class AccountApi:
    """NOTE: This class is auto generated by OpenAPI Generator.

    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, root, resource_class, sproc_client):
        self._root = root
        self._resource_name = "account"
        self._resource_class = resource_class

        if is_running_inside_stored_procedure():
            self._api_client = sproc_client
        else:
            self._api_client = ApiClient.get_default()

    @property
    def api_client(self):
        return self._api_client

    def deserialize(self, response, response_type):
        """Deserializes response into an object.

        :param response: RESTResponse object to be deserialized.
        :param response_type: class literal for
            deserialized object, or string of class name.

        :return: deserialized object.
        """
        # handle file downloading
        # save response body into a tmp file and return the instance
        if response_type == "file":
            return self.__deserialize_file(response)

        # fetch data from response object
        try:
            data = json.loads(response.data)
        except ValueError:
            data = response.data

        return self.__deserialize(data, response_type)

    def __deserialize(self, data, klass):
        """Deserializes dict, list, str into an object.

        :param data: dict, list or str.
        :param klass: class literal, or string of class name.

        :return: object.
        """
        if data is None:
            return None

        if isinstance(klass, str):
            if klass.startswith("Iterable["):
                sub_kls = re.match(r"Iterable\[(.*)]", klass).group(1)
                return [self.__deserialize(sub_data, sub_kls) for sub_data in data]

            if klass.startswith("Dict["):
                sub_kls = re.match(r"Dict\[([^,]*), (.*)]", klass).group(2)
                return {k: self.__deserialize(v, sub_kls) for k, v in data.items()}

            # convert str to class
            if klass in snowflake.core._http_requests.NATIVE_TYPES_MAPPING:
                klass = snowflake.core._http_requests.NATIVE_TYPES_MAPPING[klass]
            else:
                klass = getattr(models, klass)

        if klass in snowflake.core._http_requests.PRIMITIVE_TYPES:
            return self.__deserialize_primitive(data, klass)
        elif klass == object:  # noqa: E721
            return self.__deserialize_object(data)
        elif klass == date:
            return self.__deserialize_date(data)
        elif klass == datetime:
            return self.__deserialize_datetime(data)
        else:
            return self.__deserialize_model(data, klass)

    def __deserialize_file(self, response):
        """Deserializes body to file.

        Saves response body into a file in a temporary folder,
        using the filename from the `Content-Disposition` header if provided.

        :param response:  RESTResponse.
        :return: file path.
        """
        fd, path = tempfile.mkstemp(dir=self.api_client.configuration.temp_folder_path)
        os.close(fd)
        os.remove(path)

        content_disposition = response.getheader("Content-Disposition")
        if content_disposition:
            filename = re.search(r'filename=[\'"]?([^\'"\s]+)[\'"]?', content_disposition).group(1)
            path = os.path.join(os.path.dirname(path), filename)

        with open(path, "wb") as f:
            f.write(response.data)

        return path

    def __deserialize_primitive(self, data, klass):
        """Deserializes string to primitive type.

        :param data: str.
        :param klass: class literal.

        :return: int, long, float, str, bool.
        """
        try:
            return klass(data)
        except UnicodeEncodeError:
            return str(data)
        except TypeError:
            return data

    def __deserialize_object(self, value):
        """Return an original value.

        :return: object.
        """
        return value

    def __deserialize_date(self, string):
        """Deserializes string to date.

        :param string: str.
        :return: date.
        """
        try:
            return parse(string).date()
        except ImportError:
            return string
        except ValueError:
            raise rest.APIError(status=0, reason=f"Failed to parse `{string}` as date object")

    def __deserialize_datetime(self, string):
        """Deserializes string to datetime.

        The string should be in iso8601 datetime format.

        :param string: str.
        :return: datetime.
        """
        try:
            return parse(string)
        except ImportError:
            return string
        except ValueError:
            raise rest.APIError(status=0, reason=(f"Failed to parse `{string}` as datetime object"))

    def __deserialize_model(self, data, klass):
        """Deserializes list or dict to model.

        :param data: dict, list.
        :param klass: class literal.
        :return: model object.
        """
        return klass.from_dict(data)

    @overload
    def create_account(self, account: Account, async_req: Literal[False] = False, **kwargs) -> SuccessResponse: ...

    @overload
    def create_account(
        self, account: Account, async_req: Literal[True] = True, **kwargs
    ) -> Future[SuccessResponse]: ...

    @overload
    def create_account(
        self, account: Account, async_req: bool = False, **kwargs
    ) -> Union[SuccessResponse, Future[SuccessResponse]]: ...

    @validate_call
    def create_account(self, account: Account, **kwargs) -> Union[SuccessResponse, Future[SuccessResponse]]:
        r"""Create an account.  # noqa: E501

        Creates a account. You must provide the full account definition when creating a account.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> future = api.create_account(account, async_req=True)
        >>> result = future.result()
        :param account: (required)
        :type account: Account
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns a Future object representing the execution of the method.
        :rtype: Union[SuccessResponse, Future[SuccessResponse]]
        """
        kwargs["_return_http_data_only"] = True
        return self.create_account_with_http_info(account, **kwargs)

    @validate_call
    def create_account_with_http_info(self, account: Account, **kwargs):
        r"""Create an account.  # noqa: E501

        Creates a account. You must provide the full account definition when creating a account.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> future = api.create_account_with_http_info(account, async_req=True)
        >>> result = future.result()
        :param account: (required)
        :type account: Account
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns a Future object representing the execution of the method.
        :rtype: tuple(Union[SuccessResponse, Future[SuccessResponse]], status_code(int), headers(HTTPHeaderDict))
        """
        _params = locals()

        _all_params = ["account"]
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise _APITypeError(f"Got an unexpected keyword argument '{_key}' to method create_account")
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []

        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))

        # process the form parameters
        _form_params = []
        _files = {}

        # process the body parameter
        _body_params = None

        if _params["account"] is not None:
            _body_params = _params["account"]

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(["application/json"])

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get(
            "_content_type", self.api_client.select_header_content_type(["application/json"])
        )
        if _content_types_list:
            _header_params["Content-Type"] = _content_types_list

        # authentication setting
        _auth_settings = ["ExternalOAuth", "KeyPair", "ProgrammaticAccessToken", "SnowflakeOAuth"]

        _response_types_map = {
            "200": "SuccessResponse",
            "202": "SuccessAcceptedResponse",
            "400": "ErrorResponse",
            "401": "ErrorResponse",
            "403": "ErrorResponse",
            "404": "ErrorResponse",
            "405": "ErrorResponse",
            "408": "ErrorResponse",
            "409": "ErrorResponse",
            "410": "ErrorResponse",
            "429": "ErrorResponse",
            "500": "ErrorResponse",
            "503": "ErrorResponse",
            "504": "ErrorResponse",
        }

        return self.api_client.call_api(
            self._root,
            "/api/v2/accounts",
            "POST",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
            _deserialize_response_fn=self.deserialize,
        )

    @overload
    def list_accounts(
        self,
        like: Annotated[
            Optional[StrictStr],
            Field(
                description="Parameter to filter the command output by resource name. Uses case-insensitive pattern matching, with support for SQL wildcard characters."
            ),
        ] = None,
        show_limit: Annotated[
            Optional[Annotated[int, Field(le=10000, strict=True, ge=1)]],
            Field(description="Parameter to limit the maximum number of rows returned by a command."),
        ] = None,
        history: Annotated[
            Optional[StrictBool],
            Field(description="Optionally includes dropped accounts that have not yet been purged."),
        ] = None,
        async_req: Literal[False] = False,
        **kwargs,
    ) -> Iterable[Account]: ...

    @overload
    def list_accounts(
        self,
        like: Annotated[
            Optional[StrictStr],
            Field(
                description="Parameter to filter the command output by resource name. Uses case-insensitive pattern matching, with support for SQL wildcard characters."
            ),
        ] = None,
        show_limit: Annotated[
            Optional[Annotated[int, Field(le=10000, strict=True, ge=1)]],
            Field(description="Parameter to limit the maximum number of rows returned by a command."),
        ] = None,
        history: Annotated[
            Optional[StrictBool],
            Field(description="Optionally includes dropped accounts that have not yet been purged."),
        ] = None,
        async_req: Literal[True] = True,
        **kwargs,
    ) -> Future[Iterable[Account]]: ...

    @overload
    def list_accounts(
        self,
        like: Annotated[
            Optional[StrictStr],
            Field(
                description="Parameter to filter the command output by resource name. Uses case-insensitive pattern matching, with support for SQL wildcard characters."
            ),
        ] = None,
        show_limit: Annotated[
            Optional[Annotated[int, Field(le=10000, strict=True, ge=1)]],
            Field(description="Parameter to limit the maximum number of rows returned by a command."),
        ] = None,
        history: Annotated[
            Optional[StrictBool],
            Field(description="Optionally includes dropped accounts that have not yet been purged."),
        ] = None,
        async_req: bool = False,
        **kwargs,
    ) -> Union[Iterable[Account], Future[Iterable[Account]]]: ...

    @validate_call
    def list_accounts(
        self,
        like: Annotated[
            Optional[StrictStr],
            Field(
                description="Parameter to filter the command output by resource name. Uses case-insensitive pattern matching, with support for SQL wildcard characters."
            ),
        ] = None,
        show_limit: Annotated[
            Optional[Annotated[int, Field(le=10000, strict=True, ge=1)]],
            Field(description="Parameter to limit the maximum number of rows returned by a command."),
        ] = None,
        history: Annotated[
            Optional[StrictBool],
            Field(description="Optionally includes dropped accounts that have not yet been purged."),
        ] = None,
        **kwargs,
    ) -> Union[Iterable[Account], Future[Iterable[Account]]]:
        r"""List accounts.  # noqa: E501

        Lists the accessible accounts.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> future = api.list_accounts(like, show_limit, history, async_req=True)
        >>> result = future.result()
        :param like: Parameter to filter the command output by resource name. Uses case-insensitive pattern matching, with support for SQL wildcard characters.
        :type like: str
        :param show_limit: Parameter to limit the maximum number of rows returned by a command.
        :type show_limit: int
        :param history: Optionally includes dropped accounts that have not yet been purged.
        :type history: bool
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns a Future object representing the execution of the method.
        :rtype: Union[Iterable[Account], Future[Iterable[Account]]]
        """
        kwargs["_return_http_data_only"] = True
        return self.list_accounts_with_http_info(like, show_limit, history, **kwargs)

    @validate_call
    def list_accounts_with_http_info(
        self,
        like: Annotated[
            Optional[StrictStr],
            Field(
                description="Parameter to filter the command output by resource name. Uses case-insensitive pattern matching, with support for SQL wildcard characters."
            ),
        ] = None,
        show_limit: Annotated[
            Optional[Annotated[int, Field(le=10000, strict=True, ge=1)]],
            Field(description="Parameter to limit the maximum number of rows returned by a command."),
        ] = None,
        history: Annotated[
            Optional[StrictBool],
            Field(description="Optionally includes dropped accounts that have not yet been purged."),
        ] = None,
        **kwargs,
    ):
        r"""List accounts.  # noqa: E501

        Lists the accessible accounts.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> future = api.list_accounts_with_http_info(like, show_limit, history, async_req=True)
        >>> result = future.result()
        :param like: Parameter to filter the command output by resource name. Uses case-insensitive pattern matching, with support for SQL wildcard characters.
        :type like: str
        :param show_limit: Parameter to limit the maximum number of rows returned by a command.
        :type show_limit: int
        :param history: Optionally includes dropped accounts that have not yet been purged.
        :type history: bool
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns a Future object representing the execution of the method.
        :rtype: tuple(Union[Iterable[Account], Future[Iterable[Account]]], status_code(int), headers(HTTPHeaderDict))
        """
        _params = locals()

        _all_params = ["like", "show_limit", "history"]
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise _APITypeError(f"Got an unexpected keyword argument '{_key}' to method list_accounts")
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []

        if _params.get("like") is not None:
            _query_params.append(("like", _params["like"]))

        if _params.get("show_limit") is not None:
            _query_params.append(("showLimit", _params["show_limit"]))

        if _params.get("history") is not None:
            _query_params.append(("history", _params["history"]))

        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))

        # process the form parameters
        _form_params = []
        _files = {}

        # process the body parameter
        _body_params = None

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(["application/json"])

        # authentication setting
        _auth_settings = ["ExternalOAuth", "KeyPair", "ProgrammaticAccessToken", "SnowflakeOAuth"]

        _response_types_map = {
            "200": "Iterable[Account]",
            "202": "SuccessAcceptedResponse",
            "400": "ErrorResponse",
            "401": "ErrorResponse",
            "403": "ErrorResponse",
            "404": "ErrorResponse",
            "405": "ErrorResponse",
            "408": "ErrorResponse",
            "409": "ErrorResponse",
            "410": "ErrorResponse",
            "429": "ErrorResponse",
            "500": "ErrorResponse",
            "503": "ErrorResponse",
            "504": "ErrorResponse",
        }

        return self.api_client.call_api(
            self._root,
            "/api/v2/accounts",
            "GET",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
            _deserialize_response_fn=self.deserialize,
        )

    @overload
    def undrop_account(
        self,
        name: Annotated[str, Field(strict=True, description="Identifier (i.e. name) for the resource.")],
        async_req: Literal[False] = False,
        **kwargs,
    ) -> SuccessResponse: ...

    @overload
    def undrop_account(
        self,
        name: Annotated[str, Field(strict=True, description="Identifier (i.e. name) for the resource.")],
        async_req: Literal[True] = True,
        **kwargs,
    ) -> Future[SuccessResponse]: ...

    @overload
    def undrop_account(
        self,
        name: Annotated[str, Field(strict=True, description="Identifier (i.e. name) for the resource.")],
        async_req: bool = False,
        **kwargs,
    ) -> Union[SuccessResponse, Future[SuccessResponse]]: ...

    @validate_call
    def undrop_account(
        self, name: Annotated[str, Field(strict=True, description="Identifier (i.e. name) for the resource.")], **kwargs
    ) -> Union[SuccessResponse, Future[SuccessResponse]]:
        r"""Restore an account.  # noqa: E501

        Restores a dropped account that has not yet been permanently deleted (a dropped account that is within its grace period).  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> future = api.undrop_account(name, async_req=True)
        >>> result = future.result()
        :param name: Identifier (i.e. name) for the resource. (required)
        :type name: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns a Future object representing the execution of the method.
        :rtype: Union[SuccessResponse, Future[SuccessResponse]]
        """
        kwargs["_return_http_data_only"] = True
        return self.undrop_account_with_http_info(name, **kwargs)

    @validate_call
    def undrop_account_with_http_info(
        self, name: Annotated[str, Field(strict=True, description="Identifier (i.e. name) for the resource.")], **kwargs
    ):
        r"""Restore an account.  # noqa: E501

        Restores a dropped account that has not yet been permanently deleted (a dropped account that is within its grace period).  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> future = api.undrop_account_with_http_info(name, async_req=True)
        >>> result = future.result()
        :param name: Identifier (i.e. name) for the resource. (required)
        :type name: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns a Future object representing the execution of the method.
        :rtype: tuple(Union[SuccessResponse, Future[SuccessResponse]], status_code(int), headers(HTTPHeaderDict))
        """
        _params = locals()

        _all_params = ["name"]
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise _APITypeError(f"Got an unexpected keyword argument '{_key}' to method undrop_account")
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        if _params["name"]:
            _path_params["name"] = _params["name"]

        # process the query parameters
        _query_params = []

        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))

        # process the form parameters
        _form_params = []
        _files = {}

        # process the body parameter
        _body_params = None

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(["application/json"])

        # authentication setting
        _auth_settings = ["ExternalOAuth", "KeyPair", "ProgrammaticAccessToken", "SnowflakeOAuth"]

        _response_types_map = {
            "200": "SuccessResponse",
            "202": "SuccessAcceptedResponse",
            "400": "ErrorResponse",
            "401": "ErrorResponse",
            "403": "ErrorResponse",
            "404": "ErrorResponse",
            "405": "ErrorResponse",
            "408": "ErrorResponse",
            "409": "ErrorResponse",
            "410": "ErrorResponse",
            "429": "ErrorResponse",
            "500": "ErrorResponse",
            "503": "ErrorResponse",
            "504": "ErrorResponse",
        }

        return self.api_client.call_api(
            self._root,
            "/api/v2/accounts/{name}:undrop",
            "POST",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
            _deserialize_response_fn=self.deserialize,
        )

    @overload
    def delete_account(
        self,
        name: Annotated[str, Field(strict=True, description="Identifier (i.e. name) for the resource.")],
        grace_period_in_days: Annotated[
            StrictInt,
            Field(
                description="Specifies the number of days during which the account can be restored (“undropped”). The minimum is 3 days and the maximum is 90 days."
            ),
        ],
        if_exists: Annotated[
            Optional[StrictBool],
            Field(
                description="Parameter that specifies how to handle the request for a resource that does not exist: - `true`: The endpoint does not throw an error if the resource does not exist. It returns a 200 success response, but does not take any action on the resource. - `false`: The endpoint throws an error if the resource doesn't exist."
            ),
        ] = None,
        async_req: Literal[False] = False,
        **kwargs,
    ) -> SuccessResponse: ...

    @overload
    def delete_account(
        self,
        name: Annotated[str, Field(strict=True, description="Identifier (i.e. name) for the resource.")],
        grace_period_in_days: Annotated[
            StrictInt,
            Field(
                description="Specifies the number of days during which the account can be restored (“undropped”). The minimum is 3 days and the maximum is 90 days."
            ),
        ],
        if_exists: Annotated[
            Optional[StrictBool],
            Field(
                description="Parameter that specifies how to handle the request for a resource that does not exist: - `true`: The endpoint does not throw an error if the resource does not exist. It returns a 200 success response, but does not take any action on the resource. - `false`: The endpoint throws an error if the resource doesn't exist."
            ),
        ] = None,
        async_req: Literal[True] = True,
        **kwargs,
    ) -> Future[SuccessResponse]: ...

    @overload
    def delete_account(
        self,
        name: Annotated[str, Field(strict=True, description="Identifier (i.e. name) for the resource.")],
        grace_period_in_days: Annotated[
            StrictInt,
            Field(
                description="Specifies the number of days during which the account can be restored (“undropped”). The minimum is 3 days and the maximum is 90 days."
            ),
        ],
        if_exists: Annotated[
            Optional[StrictBool],
            Field(
                description="Parameter that specifies how to handle the request for a resource that does not exist: - `true`: The endpoint does not throw an error if the resource does not exist. It returns a 200 success response, but does not take any action on the resource. - `false`: The endpoint throws an error if the resource doesn't exist."
            ),
        ] = None,
        async_req: bool = False,
        **kwargs,
    ) -> Union[SuccessResponse, Future[SuccessResponse]]: ...

    @validate_call
    def delete_account(
        self,
        name: Annotated[str, Field(strict=True, description="Identifier (i.e. name) for the resource.")],
        grace_period_in_days: Annotated[
            StrictInt,
            Field(
                description="Specifies the number of days during which the account can be restored (“undropped”). The minimum is 3 days and the maximum is 90 days."
            ),
        ],
        if_exists: Annotated[
            Optional[StrictBool],
            Field(
                description="Parameter that specifies how to handle the request for a resource that does not exist: - `true`: The endpoint does not throw an error if the resource does not exist. It returns a 200 success response, but does not take any action on the resource. - `false`: The endpoint throws an error if the resource doesn't exist."
            ),
        ] = None,
        **kwargs,
    ) -> Union[SuccessResponse, Future[SuccessResponse]]:
        r"""Delete an account.  # noqa: E501

        Deletes the specified account. If you enable the `ifExists` parameter, the operation succeeds even if the account does not exist. Otherwise, a 404 failure is returned if the account does not exist. if the drop is unsuccessful.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> future = api.delete_account(name, grace_period_in_days, if_exists, async_req=True)
        >>> result = future.result()
        :param name: Identifier (i.e. name) for the resource. (required)
        :type name: str
        :param grace_period_in_days: Specifies the number of days during which the account can be restored (“undropped”). The minimum is 3 days and the maximum is 90 days. (required)
        :type grace_period_in_days: int
        :param if_exists: Parameter that specifies how to handle the request for a resource that does not exist: - `true`: The endpoint does not throw an error if the resource does not exist. It returns a 200 success response, but does not take any action on the resource. - `false`: The endpoint throws an error if the resource doesn't exist.
        :type if_exists: bool
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns a Future object representing the execution of the method.
        :rtype: Union[SuccessResponse, Future[SuccessResponse]]
        """
        kwargs["_return_http_data_only"] = True
        return self.delete_account_with_http_info(name, grace_period_in_days, if_exists, **kwargs)

    @validate_call
    def delete_account_with_http_info(
        self,
        name: Annotated[str, Field(strict=True, description="Identifier (i.e. name) for the resource.")],
        grace_period_in_days: Annotated[
            StrictInt,
            Field(
                description="Specifies the number of days during which the account can be restored (“undropped”). The minimum is 3 days and the maximum is 90 days."
            ),
        ],
        if_exists: Annotated[
            Optional[StrictBool],
            Field(
                description="Parameter that specifies how to handle the request for a resource that does not exist: - `true`: The endpoint does not throw an error if the resource does not exist. It returns a 200 success response, but does not take any action on the resource. - `false`: The endpoint throws an error if the resource doesn't exist."
            ),
        ] = None,
        **kwargs,
    ):
        r"""Delete an account.  # noqa: E501

        Deletes the specified account. If you enable the `ifExists` parameter, the operation succeeds even if the account does not exist. Otherwise, a 404 failure is returned if the account does not exist. if the drop is unsuccessful.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> future = api.delete_account_with_http_info(name, grace_period_in_days, if_exists, async_req=True)
        >>> result = future.result()
        :param name: Identifier (i.e. name) for the resource. (required)
        :type name: str
        :param grace_period_in_days: Specifies the number of days during which the account can be restored (“undropped”). The minimum is 3 days and the maximum is 90 days. (required)
        :type grace_period_in_days: int
        :param if_exists: Parameter that specifies how to handle the request for a resource that does not exist: - `true`: The endpoint does not throw an error if the resource does not exist. It returns a 200 success response, but does not take any action on the resource. - `false`: The endpoint throws an error if the resource doesn't exist.
        :type if_exists: bool
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns a Future object representing the execution of the method.
        :rtype: tuple(Union[SuccessResponse, Future[SuccessResponse]], status_code(int), headers(HTTPHeaderDict))
        """
        _params = locals()

        _all_params = ["name", "grace_period_in_days", "if_exists"]
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise _APITypeError(f"Got an unexpected keyword argument '{_key}' to method delete_account")
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        if _params["name"]:
            _path_params["name"] = _params["name"]

        # process the query parameters
        _query_params = []

        if _params.get("if_exists") is not None:
            _query_params.append(("ifExists", _params["if_exists"]))

        if _params.get("grace_period_in_days") is not None:
            _query_params.append(("gracePeriodInDays", _params["grace_period_in_days"]))

        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))

        # process the form parameters
        _form_params = []
        _files = {}

        # process the body parameter
        _body_params = None

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(["application/json"])

        # authentication setting
        _auth_settings = ["ExternalOAuth", "KeyPair", "ProgrammaticAccessToken", "SnowflakeOAuth"]

        _response_types_map = {
            "200": "SuccessResponse",
            "202": "SuccessAcceptedResponse",
            "400": "ErrorResponse",
            "401": "ErrorResponse",
            "403": "ErrorResponse",
            "404": "ErrorResponse",
            "405": "ErrorResponse",
            "408": "ErrorResponse",
            "409": "ErrorResponse",
            "410": "ErrorResponse",
            "429": "ErrorResponse",
            "500": "ErrorResponse",
            "503": "ErrorResponse",
            "504": "ErrorResponse",
        }

        return self.api_client.call_api(
            self._root,
            "/api/v2/accounts/{name}",
            "DELETE",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
            _deserialize_response_fn=self.deserialize,
        )



"""Manages Snowflake secret."""

from ._generated.models import (
    CloudProviderTokenSecret,
    GenericStringSecret,
    JwtKeyPairSecret,
    Oauth2Secret,
    PasswordSecret,
    Secret,
    SymmetricKeySecret,
)
from ._secret import SecretCollection, SecretResource


__all__ = [
    "SecretResource",
    "SecretCollection",
    "CloudProviderTokenSecret",
    "GenericStringSecret",
    "JwtKeyPairSecret",
    "Oauth2Secret",
    "PasswordSecret",
    "Secret",
    "SymmetricKeySecret",
]



import datetime

from unittest.mock import Mock

import pytest

from snowflake.core.task.context import TaskContext


@pytest.mark.snowpark
def test_task_context(fake_root):
    from snowflake.snowpark.exceptions import SnowparkSQLException

    task_context = TaskContext(fake_root)
    mock_result_str = Mock()

    def side_effect_collect_str():
        return [["abc"]]

    mock_result_str.collect.side_effect = side_effect_collect_str
    mock_result_datetime = Mock()

    def side_effect_collect_datetime():
        return [[datetime.datetime(2020, 1, 1)]]

    mock_result_datetime.collect.side_effect = side_effect_collect_datetime
    mock_sql_result = {
        "select to_char(system$task_runtime_info('CURRENT_ROOT_TASK_NAME'))": mock_result_str,
        "select to_char(system$task_runtime_info('CURRENT_ROOT_TASK_UUID'))": mock_result_str,
        "select to_char(system$task_runtime_info('LAST_SUCCESSFUL_TASK_GRAPH_RUN_GROUP_ID'))": mock_result_str,
        "select to_char(system$task_runtime_info('CURRENT_TASK_GRAPH_RUN_GROUP_ID'))": mock_result_str,
        "select to_timestamp(system$task_runtime_info('CURRENT_TASK_GRAPH_ORIGINAL_SCHEDULED_TIMESTAMP'))": mock_result_datetime,
        "select to_timestamp(system$task_runtime_info('LAST_SUCCESSFUL_TASK_GRAPH_ORIGINAL_SCHEDULED_TIMESTAMP'))": mock_result_datetime,
    }

    def side_effect_sql(k):
        return mock_sql_result[k]

    fake_root.sql.side_effect = side_effect_sql

    assert task_context.get_current_root_task_name() == "abc"
    assert task_context.get_current_root_task_uuid() == "abc"
    assert task_context.get_current_task_graph_run_group_id() == "abc"
    assert task_context.get_last_successful_task_graph_run_group_id() == "abc"
    assert task_context.get_current_task_graph_original_schedule() == datetime.datetime(2020, 1, 1)
    assert task_context.get_last_successful_task_graph_original_schedule() == datetime.datetime(2020, 1, 1)

    def side_effect_call(sp_name, para=None):
        if sp_name == "system$set_return_value":
            return None
        if sp_name == "system$get_predecessor_return_value":
            return para
        if sp_name == "system$current_user_task_name":
            return "current_task_name"
        if sp_name == "SYSTEM$GET_TASK_GRAPH_CONFIG":
            return '{"a": "b", "c": 1}'

    fake_root.call.side_effect = side_effect_call
    assert task_context.get_current_task_name() == "current_task_name"
    assert task_context.get_current_task_name() == "current_task_name"  # second time get the cache
    assert task_context.get_current_task_short_name() == "current_task_name"
    assert task_context.get_predecessor_return_value("task1") == "TASK1"
    assert task_context.set_return_value("rv") is None
    assert task_context.get_task_graph_config() == {"a": "b", "c": 1}
    assert task_context.get_task_graph_config_property("c") == 1

    def side_effect_call_none(sp_name, para=None):
        if sp_name == "SYSTEM$GET_TASK_GRAPH_CONFIG":
            return ""

    fake_root.call.side_effect = side_effect_call_none
    assert task_context.get_task_graph_config() is None
    with pytest.raises(ValueError) as ve:
        task_context.get_runtime_info("")
    assert ve.match("`property_name` must be an non-empty str.")

    def side_effect_raise_null_result(*args):
        raise SnowparkSQLException("NULL result in a non-nullable column")

    fake_root.sql.side_effect = side_effect_raise_null_result
    assert task_context.get_runtime_info("CURRENT_TASK_GRAPH_ORIGINAL_SCHEDULED_TIMESTAMP") is None

    def side_effect_raise_other_error(*args):
        raise SnowparkSQLException("Some errors")

    fake_root.sql.side_effect = side_effect_raise_other_error
    with pytest.raises(SnowparkSQLException) as sse:
        task_context.get_runtime_info("CURRENT_TASK_GRAPH_ORIGINAL_SCHEDULED_TIMESTAMP")
    assert sse.match("Some error")


def test_get_current_task_short_name(fake_root):
    task_context = TaskContext(fake_root)

    task_context.get_current_root_task_name = Mock(return_value="root")
    task_context.get_current_task_name = Mock(return_value="root$abc")
    assert task_context.get_current_task_name() == "root$abc"
    assert task_context.get_current_task_short_name() == "abc"
    task_context.get_current_task_name.return_value = "no_root"
    assert task_context.get_current_task_short_name() == "no_root"



import pytest

from snowflake.core.artifact_repository import ArtifactRepository
from snowflake.core.exceptions import NotFoundError
from tests.integ.utils import random_string


def test_rename_artifact_repository(artifact_repositories, temp_pypi_api_integration):
    """Test basic rename functionality for artifact repositories."""
    original_name = random_string(10, "test_original_artifact_repo_")
    new_name = random_string(10, "test_renamed_artifact_repo_")

    ar_def = ArtifactRepository(
        name=original_name,
        type="PIP",
        api_integration=temp_pypi_api_integration,
        comment="Original artifact repository for rename test",
    )
    ar_handle = artifact_repositories.create(ar_def)

    try:
        fetched_ar = ar_handle.fetch()
        assert fetched_ar.name == original_name.upper()

        ar_handle.rename(new_name)

        fetched_ar = ar_handle.fetch()
        assert fetched_ar.name == new_name.upper()
        assert fetched_ar.schema_name == artifact_repositories.schema.name.upper()
        assert fetched_ar.database_name == artifact_repositories.database.name.upper()

        with pytest.raises(NotFoundError):
            artifact_repositories[original_name].fetch()

    finally:
        ar_handle.drop(if_exists=True)


def test_rename_nonexistent_artifact_repository(artifact_repositories):
    """Test renaming a non-existent artifact repository raises NotFoundError."""
    nonexistent_name = random_string(10, "test_nonexistent_artifact_repo_")
    new_name = random_string(10, "test_new_artifact_repo_")

    with pytest.raises(NotFoundError):
        artifact_repositories[nonexistent_name].rename(new_name)


def test_rename_artifact_repository_with_if_exists(artifact_repositories):
    """Test renaming with if_exists=True doesn't error for non-existent repositories."""
    nonexistent_name = random_string(10, "test_nonexistent_artifact_repo_")
    new_name = random_string(10, "test_new_artifact_repo_")

    artifact_repositories[nonexistent_name].rename(new_name, if_exists=True)


def test_rename_artifact_repository_cross_schema(artifact_repositories, temp_schema, temp_pypi_api_integration):
    """Test renaming artifact repository across schemas."""
    original_name = random_string(10, "test_cross_schema_artifact_repo_")
    new_name = random_string(10, "test_renamed_cross_schema_artifact_repo_")

    ar_def = ArtifactRepository(
        name=original_name,
        type="PIP",
        api_integration=temp_pypi_api_integration,
        comment="Cross-schema rename test repository",
    )
    ar_handle = artifact_repositories.create(ar_def)

    try:
        ar_handle.rename(new_name, target_schema=temp_schema.name, target_database=ar_handle.database.name)

        fetched_ar = ar_handle.fetch()
        assert fetched_ar.name.upper() == new_name.upper()
        assert fetched_ar.schema_name.upper() == temp_schema.name.upper()
        assert fetched_ar.database_name.upper() == ar_handle.database.name.upper()

        with pytest.raises(NotFoundError):
            artifact_repositories[original_name].fetch()

    finally:
        ar_handle.drop(if_exists=True)


def test_rename_artifact_repository_cross_database(artifact_repositories, temp_db, temp_pypi_api_integration):
    """Test renaming artifact repository across databases."""
    original_name = random_string(10, "test_cross_db_artifact_repo_")
    new_name = random_string(10, "test_renamed_cross_db_artifact_repo_")

    ar_def = ArtifactRepository(
        name=original_name,
        type="PIP",
        api_integration=temp_pypi_api_integration,
        comment="Cross-database rename test repository",
    )
    ar_handle = artifact_repositories.create(ar_def)

    try:
        ar_handle.rename(new_name, target_database=temp_db.name, target_schema="PUBLIC")

        fetched_ar = ar_handle.fetch()
        assert fetched_ar.name.upper() == new_name.upper()
        assert fetched_ar.database_name.upper() == temp_db.name.upper()
        assert fetched_ar.schema_name.upper() == "PUBLIC"

        with pytest.raises(NotFoundError):
            artifact_repositories[original_name].fetch()

    finally:
        ar_handle.drop(if_exists=True)



import pytest as pytest

from snowflake.core.exceptions import NotFoundError


def test_drop_and_undrop(dynamic_table_handle, dynamic_tables):
    dynamic_tables["dummy___table"].drop(if_exists=True)
    dynamic_table_handle.drop()
    with pytest.raises(NotFoundError):
        dynamic_table_handle.fetch()
    dynamic_table_handle.drop(if_exists=True)
    dynamic_table_handle.undrop()
    dynamic_table_handle.fetch()



"""
Snowflake Grant API.

The Snowflake Grant API is a REST API that you can use to show or manage privileges that have been provided to users and roles in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from datetime import datetime
from typing import Any, List, Optional

from pydantic import BaseModel, ConfigDict, StrictBool, StrictStr


class Grant(BaseModel):
    """A model object representing the Grant resource.

    Constructs an object of type Grant with the provided properties.

    Parameters
    __________
    privileges : list[str], optional
        Privilege type
    grant_option : bool, optional
        Can grantee pass this privilege down?
    created_on : datetime, optional

    grantee_type : str, optional
        Entity type being granted to
    grantee_name : str, optional
        Specific name of object being granted to
    securable_type : str, optional
        Type of object granted on
    securable_name : str, optional
        Name of specific object granted on (not name of privilege!)
    granted_by_role_type : str, optional
        Type of role that granted this privilege to this grantee
    granted_by_name : str, optional
        The role that granted this privilege to this grantee
    """

    privileges: Optional[List[StrictStr]] = None

    grant_option: Optional[StrictBool] = None

    created_on: Optional[datetime] = None

    grantee_type: Optional[StrictStr] = None

    grantee_name: Optional[StrictStr] = None

    securable_type: Optional[StrictStr] = None

    securable_name: Optional[StrictStr] = None

    granted_by_role_type: Optional[StrictStr] = None

    granted_by_name: Optional[StrictStr] = None

    __properties = [
        "privileges",
        "grant_option",
        "created_on",
        "grantee_type",
        "grantee_name",
        "securable_type",
        "securable_name",
        "granted_by_role_type",
        "granted_by_name",
    ]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Grant:
        """Create an instance of Grant from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> Grant:
        """Create an instance of Grant from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return Grant.model_validate(obj)

        _obj = Grant.model_validate(
            {
                "privileges": obj.get("privileges"),
                "grant_option": obj.get("grant_option"),
                "created_on": obj.get("created_on"),
                "grantee_type": obj.get("grantee_type"),
                "grantee_name": obj.get("grantee_name"),
                "securable_type": obj.get("securable_type"),
                "securable_name": obj.get("securable_name"),
                "granted_by_role_type": obj.get("granted_by_role_type"),
                "granted_by_name": obj.get("granted_by_name"),
            }
        )

        return _obj


class GrantModel:
    def __init__(
        self,  # optional properties
        privileges: Optional[list[str]] = None,
        grant_option: Optional[bool] = None,
        created_on: Optional[datetime] = None,
        grantee_type: Optional[str] = None,
        grantee_name: Optional[str] = None,
        securable_type: Optional[str] = None,
        securable_name: Optional[str] = None,
        granted_by_role_type: Optional[str] = None,
        granted_by_name: Optional[str] = None,
    ):
        """A model object representing the Grant resource.

        Constructs an object of type Grant with the provided properties.

        Parameters
        __________
        privileges : list[str], optional
            Privilege type
        grant_option : bool, optional
            Can grantee pass this privilege down?
        created_on : datetime, optional

        grantee_type : str, optional
            Entity type being granted to
        grantee_name : str, optional
            Specific name of object being granted to
        securable_type : str, optional
            Type of object granted on
        securable_name : str, optional
            Name of specific object granted on (not name of privilege!)
        granted_by_role_type : str, optional
            Type of role that granted this privilege to this grantee
        granted_by_name : str, optional
            The role that granted this privilege to this grantee
        """
        self.privileges = privileges
        self.grant_option = grant_option
        self.created_on = created_on
        self.grantee_type = grantee_type
        self.grantee_name = grantee_name
        self.securable_type = securable_type
        self.securable_name = securable_name
        self.granted_by_role_type = granted_by_role_type
        self.granted_by_name = granted_by_name

    __properties = [
        "privileges",
        "grant_option",
        "created_on",
        "grantee_type",
        "grantee_name",
        "securable_type",
        "securable_name",
        "granted_by_role_type",
        "granted_by_name",
    ]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return Grant(
            privileges=self.privileges,
            grant_option=self.grant_option,
            created_on=self.created_on,
            grantee_type=self.grantee_type,
            grantee_name=self.grantee_name,
            securable_type=self.securable_type,
            securable_name=self.securable_name,
            granted_by_role_type=self.granted_by_role_type,
            granted_by_name=self.granted_by_name,
        )

    @classmethod
    def _from_model(cls, model) -> GrantModel:
        return GrantModel(
            privileges=model.privileges,
            grant_option=model.grant_option,
            created_on=model.created_on,
            grantee_type=model.grantee_type,
            grantee_name=model.grantee_name,
            securable_type=model.securable_type,
            securable_name=model.securable_name,
            granted_by_role_type=model.granted_by_role_type,
            granted_by_name=model.granted_by_name,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a Grant.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the Grant object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> GrantModel:
        """Create an instance of Grant from a dict.

        This method constructs a Grant object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        Grant
            A Grant object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(Grant.from_dict(obj))


Grant._model_class = GrantModel



# coding: utf-8

# flake8: noqa
"""
Snowflake Network Policy API.

The Snowflake Network Policy API is a REST API that you can use to access, update, and perform certain actions on Network Policy resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import absolute_import

# import models into model package
from snowflake.core.network_policy._generated.models.error_response import ErrorResponse
from snowflake.core.network_policy._generated.models.network_policy import NetworkPolicy
from snowflake.core.network_policy._generated.models.success_accepted_response import SuccessAcceptedResponse
from snowflake.core.network_policy._generated.models.success_response import SuccessResponse
from snowflake.core.network_policy._generated.models.tag_assignment import TagAssignment
from snowflake.core.network_policy._generated.models.tag_reference import TagReference

__all__ = [
    "ErrorResponse",
    "NetworkPolicy",
    "SuccessAcceptedResponse",
    "SuccessResponse",
    "TagAssignment",
    "TagReference",
]
